#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: ec2_vpc_endpoint_info
short_description: Retrieves AWS VPC endpoints details using AWS methods
version_added: 1.0.0
description:
  - Gets various details related to AWS VPC endpoints.
options:
  vpc_endpoint_ids:
    description:
      - The IDs of specific endpoints to retrieve the details of.
    type: list
    elements: str
  filters:
    description:
      - A dict of filters to apply. Each dict item consists of a filter key and a filter value.
        See U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeVpcEndpoints.html)
        for possible filters.
    type: dict
    default: {}
author:
  - Karen Cheng (@Etherdaemon)
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
notes:
  - Support for the I(query) parameter was dropped in release 6.0.0.  This module now only queries
    for endpoints. Information about endpoint services can be retrieved using the
    M(amazon.aws.ec2_vpc_endpoint_service_info) module.
"""

EXAMPLES = r"""
# Simple example of listing all support AWS services for VPC endpoints
- name: Get all endpoints in ap-southeast-2 region
  amazon.aws.ec2_vpc_endpoint_info:
    region: ap-southeast-2
  register: existing_endpoints

- name: Get all endpoints with specific filters
  amazon.aws.ec2_vpc_endpoint_info:
    region: ap-southeast-2
    filters:
      vpc-id:
        - vpc-12345678
        - vpc-87654321
      vpc-endpoint-state:
        - available
        - pending
  register: existing_endpoints

- name: Get details on specific endpoint
  amazon.aws.ec2_vpc_endpoint_info:
    region: ap-southeast-2
    vpc_endpoint_ids:
      - vpce-12345678
  register: endpoint_details
"""

RETURN = r"""
vpc_endpoints:
  description:
    - A list of matching endpoints.
  returned: always
  type: list
  elements: dict
  contains:
    creation_timestamp:
      description: The date and time that the endpoint was created.
      returned: always
      type: str
    dns_entries:
      description: List of DNS entires for the endpoint.
      returned: always
      type: list
      elements: dict
      contains:
        dns_name:
          description: The DNS name.
          returned: always
          type: str
        hosted_zone_id:
          description: The ID of the private hosted zone.
          type: str
    groups:
      description: List of security groups associated with the network interface.
      returned: always
      type: list
      elements: dict
      contains:
        group_id:
          description: The ID of the security group.
          returned: always
          type: str
        group_name:
          description: The name of the security group.
          returned: always
          type: str
    ip_address_type:
      description: The IP address type for the endpoint.
      type: str
    network_interface_ids:
      description: List of network interfaces for the endpoint.
      returned: always
      type: list
      elements: str
    owner_id:
      description: The ID of the AWS account that owns the endpoint.
      returned: always
      type: str
    policy_document:
      description: The policy document associated with the endpoint.
      returned: always
      type: str
    private_dns_enabled:
      description: Indicates whether the VPC is associated with a private hosted zone.
      returned: always
      type: bool
    requester_managed:
      description: Indicated whether the endpoint is being managed by its service.
      returned: always
      type: bool
    route_table_ids:
      description: List of route table IDs associated with the endpoint.
      returned: always
      type: list
      elements: str
    service_name:
      description: The name of the service to which the endpoint is associated.
      returned: always
      type: str
    state:
      description: The state of the endpoint.
      returned: always
      type: str
    subnet_ids:
      description: List of subnets associated with the endpoint.
      returned: always
      type: list
    tags:
      description: List of tags associated with the endpoint.
      returned: always
      type: list
      elements: dict
    vpc_endpoint_id:
      description: The ID of the endpoint.
      returned: always
      type: str
    vpc_endpoint_type:
      description: The type of endpoint.
      returned: always
      type: str
    vpc_id:
      description: The ID of the VPC.
      returned: always
      type: str
  sample:
    vpc_endpoints:
    - creation_timestamp: "2017-02-16T11:06:48+00:00"
      policy_document: >
        "{\"Version\":\"2012-10-17\",\"Id\":\"Policy1450910922815\",
        \"Statement\":[{\"Sid\":\"Stmt1450910920641\",\"Effect\":\"Allow\",
        \"Principal\":\"*\",\"Action\":\"s3:*\",\"Resource\":[\"arn:aws:s3:::*/*\",\"arn:aws:s3:::*\"]}]}"
      route_table_ids:
      - rtb-abcd1234
      service_name: "com.amazonaws.ap-southeast-2.s3"
      state: "available"
      vpc_endpoint_id: "vpce-abbad0d0"
      vpc_id: "vpc-1111ffff"
"""

from typing import Any
from typing import Dict
from typing import List

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import normalize_boto3_result
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AnsibleEC2Error
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_vpc_endpoints
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list


def get_endpoints(client, module: AnsibleAWSModule) -> Dict[str, List[Dict[str, Any]]]:
    results = list()
    params = dict()
    params["Filters"] = ansible_dict_to_boto3_filter_list(module.params.get("filters"))
    if module.params.get("vpc_endpoint_ids"):
        params["VpcEndpointIds"] = module.params.get("vpc_endpoint_ids")
    try:
        results = describe_vpc_endpoints(client, **params)
        if not results:
            module.exit_json(
                msg=f"VpcEndpoint {module.params.get('vpc_endpoint_ids')} does not exist", vpc_endpoints=[]
            )
        results = normalize_boto3_result(results)
    except AnsibleEC2Error as e:
        module.fail_json_aws(e, msg="Failed to get endpoints")

    return dict(vpc_endpoints=[camel_dict_to_snake_dict(result) for result in results])


def main() -> None:
    argument_spec = dict(
        filters=dict(default={}, type="dict"),
        vpc_endpoint_ids=dict(type="list", elements="str"),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    # Validate Requirements
    try:
        connection = module.client("ec2")
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS")

    results = get_endpoints(connection, module)

    module.exit_json(**results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_firmware_upgrades
short_description: Resource module for networks _firmware _upgrades
description:
  - Manage operation update of the resource networks _firmware _upgrades.
  - Update firmware upgrade information for a network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  products:
    description: Contains information about the network to update.
    suboptions:
      appliance:
        description: The network device to be updated.
        suboptions:
          nextUpgrade:
            description: The pending firmware upgrade if it exists.
            suboptions:
              time:
                description: The time of the last successful upgrade.
                type: str
              toVersion:
                description: The version to be updated to.
                suboptions:
                  id:
                    description: The version ID.
                    type: str
                type: dict
            type: dict
          participateInNextBetaRelease:
            description: Whether or not the network wants beta firmware.
            type: bool
        type: dict
      camera:
        description: The network device to be updated.
        suboptions:
          nextUpgrade:
            description: The pending firmware upgrade if it exists.
            suboptions:
              time:
                description: The time of the last successful upgrade.
                type: str
              toVersion:
                description: The version to be updated to.
                suboptions:
                  id:
                    description: The version ID.
                    type: str
                type: dict
            type: dict
          participateInNextBetaRelease:
            description: Whether or not the network wants beta firmware.
            type: bool
        type: dict
      cellularGateway:
        description: The network device to be updated.
        suboptions:
          nextUpgrade:
            description: The pending firmware upgrade if it exists.
            suboptions:
              time:
                description: The time of the last successful upgrade.
                type: str
              toVersion:
                description: The version to be updated to.
                suboptions:
                  id:
                    description: The version ID.
                    type: str
                type: dict
            type: dict
          participateInNextBetaRelease:
            description: Whether or not the network wants beta firmware.
            type: bool
        type: dict
      secureConnect:
        description: The network device to be updated.
        suboptions:
          nextUpgrade:
            description: The pending firmware upgrade if it exists.
            suboptions:
              time:
                description: The time of the last successful upgrade.
                type: str
              toVersion:
                description: The version to be updated to.
                suboptions:
                  id:
                    description: The version ID.
                    type: str
                type: dict
            type: dict
          participateInNextBetaRelease:
            description: Whether or not the network wants beta firmware.
            type: bool
        type: dict
      sensor:
        description: The network device to be updated.
        suboptions:
          nextUpgrade:
            description: The pending firmware upgrade if it exists.
            suboptions:
              time:
                description: The time of the last successful upgrade.
                type: str
              toVersion:
                description: The version to be updated to.
                suboptions:
                  id:
                    description: The version ID.
                    type: str
                type: dict
            type: dict
          participateInNextBetaRelease:
            description: Whether or not the network wants beta firmware.
            type: bool
        type: dict
      switch:
        description: The network device to be updated.
        suboptions:
          nextUpgrade:
            description: The pending firmware upgrade if it exists.
            suboptions:
              time:
                description: The time of the last successful upgrade.
                type: str
              toVersion:
                description: The version to be updated to.
                suboptions:
                  id:
                    description: The version ID.
                    type: str
                type: dict
            type: dict
          participateInNextBetaRelease:
            description: Whether or not the network wants beta firmware.
            type: bool
        type: dict
      switchCatalyst:
        description: The network device to be updated.
        suboptions:
          nextUpgrade:
            description: The pending firmware upgrade if it exists.
            suboptions:
              time:
                description: The time of the last successful upgrade.
                type: str
              toVersion:
                description: The version to be updated to.
                suboptions:
                  id:
                    description: The version ID.
                    type: str
                type: dict
            type: dict
          participateInNextBetaRelease:
            description: Whether or not the network wants beta firmware.
            type: bool
        type: dict
      wireless:
        description: The network device to be updated.
        suboptions:
          nextUpgrade:
            description: The pending firmware upgrade if it exists.
            suboptions:
              predownload:
                description: Predownload settings for the firmware upgrade.
                suboptions:
                  enabled:
                    description: Whether or not the network devices will predownload
                      the firmware image in advance of the actual upgrade.
                    type: bool
                type: dict
              time:
                description: The time of the last successful upgrade.
                type: str
              toVersion:
                description: The version to be updated to.
                suboptions:
                  id:
                    description: The version ID.
                    type: str
                type: dict
            type: dict
          participateInNextBetaRelease:
            description: Whether or not the network wants beta firmware.
            type: bool
        type: dict
      wirelessController:
        description: The network device to be updated.
        suboptions:
          nextUpgrade:
            description: The pending firmware upgrade if it exists.
            suboptions:
              time:
                description: The time of the last successful upgrade.
                type: str
              toVersion:
                description: The version to be updated to.
                suboptions:
                  id:
                    description: The version ID.
                    type: str
                type: dict
            type: dict
          participateInNextBetaRelease:
            description: Whether or not the network wants beta firmware.
            type: bool
        type: dict
    type: dict
  timezone:
    description: The timezone for the network.
    type: str
  upgradeWindow:
    description: Upgrade window for devices in network.
    suboptions:
      dayOfWeek:
        description: Day of the week.
        type: str
      hourOfDay:
        description: Hour of the day.
        type: str
    type: dict
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for networks updateNetworkFirmwareUpgrades
    description: Complete reference of the updateNetworkFirmwareUpgrades API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-firmware-upgrades
notes:
  - SDK Method used are
    networks.Networks.update_network_firmware_upgrades,
  - Paths used are
    put /networks/{networkId}/firmwareUpgrades,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_firmware_upgrades:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    networkId: string
    products:
      appliance:
        nextUpgrade:
          time: '2019-03-17T17:22:52Z'
          toVersion:
            id: '1001'
        participateInNextBetaRelease: false
      camera:
        nextUpgrade:
          time: '2019-03-17T17:22:52Z'
          toVersion:
            id: '1003'
        participateInNextBetaRelease: false
      cellularGateway:
        nextUpgrade:
          time: '2019-03-17T17:22:52Z'
          toVersion:
            id: '1004'
        participateInNextBetaRelease: false
      secureConnect:
        nextUpgrade:
          time: '2019-03-17T17:22:52Z'
          toVersion:
            id: '1008'
        participateInNextBetaRelease: false
      sensor:
        nextUpgrade:
          time: '2019-03-17T17:22:52Z'
          toVersion:
            id: '1005'
        participateInNextBetaRelease: false
      switch:
        nextUpgrade:
          time: '2019-03-17T17:22:52Z'
          toVersion:
            id: '1002'
        participateInNextBetaRelease: false
      switchCatalyst:
        nextUpgrade:
          time: '2019-03-17T17:22:52Z'
          toVersion:
            id: '1234'
        participateInNextBetaRelease: false
      wireless:
        nextUpgrade:
          predownload:
            enabled: false
          time: '2019-03-17T17:22:52Z'
          toVersion:
            id: '1000'
        participateInNextBetaRelease: false
      wirelessController:
        nextUpgrade:
          time: '2019-03-17T17:22:52Z'
          toVersion:
            id: '1006'
        participateInNextBetaRelease: false
    timezone: America/Los_Angeles
    upgradeWindow:
      dayOfWeek: sun
      hourOfDay: '4:00'
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "upgradeWindow": {
        "dayOfWeek": "string",
        "hourOfDay": "string"
      },
      "timezone": "string",
      "products": {
        "wireless": {
          "currentVersion": {
            "id": "string",
            "firmware": "string",
            "shortName": "string",
            "releaseType": "string",
            "releaseDate": "string"
          },
          "lastUpgrade": {
            "time": "string",
            "fromVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            },
            "toVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          },
          "nextUpgrade": {
            "time": "string",
            "predownload": {
              "enabled": true
            },
            "toVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          },
          "isUpgradeAvailable": true,
          "availableVersions": [
            {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          ],
          "participateInNextBetaRelease": true
        },
        "appliance": {
          "currentVersion": {
            "id": "string",
            "firmware": "string",
            "shortName": "string",
            "releaseType": "string",
            "releaseDate": "string"
          },
          "lastUpgrade": {
            "time": "string",
            "fromVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            },
            "toVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          },
          "nextUpgrade": {
            "time": "string",
            "toVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          },
          "isUpgradeAvailable": true,
          "availableVersions": [
            {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          ],
          "participateInNextBetaRelease": true
        },
        "switch": {
          "currentVersion": {
            "id": "string",
            "firmware": "string",
            "shortName": "string",
            "releaseType": "string",
            "releaseDate": "string"
          },
          "lastUpgrade": {
            "time": "string",
            "fromVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            },
            "toVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          },
          "nextUpgrade": {
            "time": "string",
            "toVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          },
          "isUpgradeAvailable": true,
          "availableVersions": [
            {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          ],
          "participateInNextBetaRelease": true
        },
        "camera": {
          "currentVersion": {
            "id": "string",
            "firmware": "string",
            "shortName": "string",
            "releaseType": "string",
            "releaseDate": "string"
          },
          "lastUpgrade": {
            "time": "string",
            "fromVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            },
            "toVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          },
          "nextUpgrade": {
            "time": "string",
            "toVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          },
          "isUpgradeAvailable": true,
          "availableVersions": [
            {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          ],
          "participateInNextBetaRelease": true
        },
        "cellularGateway": {
          "currentVersion": {
            "id": "string",
            "firmware": "string",
            "shortName": "string",
            "releaseType": "string",
            "releaseDate": "string"
          },
          "lastUpgrade": {
            "time": "string",
            "fromVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            },
            "toVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          },
          "nextUpgrade": {
            "time": "string",
            "toVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          },
          "isUpgradeAvailable": true,
          "availableVersions": [
            {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          ],
          "participateInNextBetaRelease": true
        },
        "sensor": {
          "currentVersion": {
            "id": "string",
            "firmware": "string",
            "shortName": "string",
            "releaseType": "string",
            "releaseDate": "string"
          },
          "lastUpgrade": {
            "time": "string",
            "fromVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            },
            "toVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          },
          "nextUpgrade": {
            "time": "string",
            "toVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          },
          "isUpgradeAvailable": true,
          "availableVersions": [
            {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          ],
          "participateInNextBetaRelease": true
        },
        "wirelessController": {
          "currentVersion": {
            "id": "string",
            "firmware": "string",
            "shortName": "string",
            "releaseType": "string",
            "releaseDate": "string"
          },
          "lastUpgrade": {
            "time": "string",
            "fromVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            },
            "toVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          },
          "nextUpgrade": {
            "time": "string",
            "toVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          },
          "isUpgradeAvailable": true,
          "availableVersions": [
            {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          ],
          "participateInNextBetaRelease": true
        },
        "secureConnect": {
          "currentVersion": {
            "id": "string",
            "firmware": "string",
            "shortName": "string",
            "releaseType": "string",
            "releaseDate": "string"
          },
          "lastUpgrade": {
            "time": "string",
            "fromVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            },
            "toVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          },
          "nextUpgrade": {
            "time": "string",
            "toVersion": {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          },
          "isUpgradeAvailable": true,
          "availableVersions": [
            {
              "id": "string",
              "firmware": "string",
              "shortName": "string",
              "releaseType": "string",
              "releaseDate": "string"
            }
          ],
          "participateInNextBetaRelease": true
        }
      }
    }
"""



#!/usr/bin/python

# Copyright: (c) 2023, Eric Girard <@netgirard>
# Copyright: (c) 2024, Gaspard Micol (@gmicol) <gmicol@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_interface_policy_storm_control
short_description: Manage Storm Control interface policies (stormctrl:IfPol)
description:
- Manage Storm Control interface policies on Cisco ACI fabrics.
options:
  storm_control_policy:
    description:
    - The Storm Control interface policy name.
    type: str
    aliases: [ storm_control, storm_control_name, name ]
  description:
    description:
    - The description for the Storm interface policy name.
    type: str
    aliases: [ descr ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
  storm_control_action:
    description:
    - The storm control action to take when triggered.
    type: str
    choices: [drop, shutdown]
  storm_control_soak_action:
    description:
    - The number of instances before triggering shutdown.
    type: int
  all_types_configuration:
    description:
    - The rates configuration for all packets type.
    type: dict
    aliases: [ all_types ]
    suboptions:
      rate:
        description:
        - The rate for all packet types.
        type: str
      burst_rate:
        description:
        - The burst rate of all packet types.
        type: str
      rate_type:
        description:
        - The type of rate of all packet types.
        - Choice between percentage of the bandiwth C(percentage) or packet per second C(pps)
        type: str
        choices: [ percentage, pps ]
        required: true
  broadcast_configuration:
    description:
    - The rates configuration of broadcast packets.
    type: dict
    aliases: [ broadcast ]
    suboptions:
      rate:
        description:
        - The rate for broadcast packets.
        type: str
      burst_rate:
        description:
        - The burst rate of broadcast packets.
        type: str
      rate_type:
        description:
        - The type of rate of all packet types.
        - Choice between percentage of the bandiwth C(percentage) or packet per second C(pps)
        type: str
        choices: [ percentage, pps ]
        required: true
  multicast_configuration:
    description:
    - The rates configuration of multicast packets.
    type: dict
    aliases: [ multicast ]
    suboptions:
      rate:
        description:
        - The rate for multicast packets.
        type: str
      burst_rate:
        description:
        - The burst rate of multicast packets.
        type: str
      rate_type:
        description:
        - The type of rate of all packet types.
        - Choice between percentage of the bandiwth C(percentage) or packet per second C(pps)
        type: str
        choices: [ percentage, pps ]
        required: true
  unicast_configuration:
    description:
    - The rates configuration of unicast packets.
    type: dict
    aliases: [ unicast ]
    suboptions:
      rate:
        description:
        - The rate for unicast packets.
        type: str
      burst_rate:
        description:
        - The burst rate of unicast packets.
        type: str
      rate_type:
        description:
        - The type of rate of all packet types.
        - Choice between percentage of the bandiwth C(percentage) or packet per second C(pps)
        type: str
        choices: [ percentage, pps ]
        required: true
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(stormctrl:IfPol).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Eric Girard (@netgirard)
- Gaspard Micol (@gmicol)
"""

EXAMPLES = r"""
- name: Add a new Storm Control Interface Policy
  cisco.aci.aci_interface_policy_storm_control:
    host: apic
    username: admin
    password: SomeSecretPassword
    storm_control_policy: my_storm_control_policy
    description: My Storm Control Policy
    all_types_configuration:
      rate: 80
      burst_rate: 100
      rate_type: percentage
    storm_control_action: shutdown
    storm_control_soak_action: 5
    state: present
  delegate_to: localhost

- name: Query a Storm Control Interface Policy
  cisco.aci.aci_interface_policy_storm_control:
    host: apic
    username: admin
    password: SomeSecretPassword
    storm_control_policy: my_storm_control_policy
    state: query
  delegate_to: localhost

- name: Query all Storm Control Interface Policies
  cisco.aci.aci_interface_policy_storm_control:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost

- name: Delete a Storm Control Interface Policy
  cisco.aci.aci_interface_policy_storm_control:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    storm_control_policy: my_storm_control_policy
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "cdpIfPol": {
                "attributes": {
                    "adminSt": "disabled",
                    "annotation": "",
                    "descr": "Ansible Created CDP Test Policy",
                    "dn": "uni/infra/cdpIfP-Ansible_CDP_Test_Policy",
                    "name": "Ansible_CDP_Test_Policy",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import (
    ACIModule,
    aci_argument_spec,
    aci_annotation_spec,
    aci_owner_spec,
    storm_control_policy_rate_spec,
)


def get_rates_configuration(module, configuration, percentage, pps, burst_percentage, burst_pps):
    if configuration is None:
        return {}
    rate = configuration.get("rate")
    burst_rate = configuration.get("burst_rate")
    rate_type = configuration.get("rate_type")

    if rate_type == "percentage":
        for rate_name, rate_value in dict(rate=rate, burst_rate=burst_rate).items():
            if rate_value is None or not (0 <= float(rate_value) <= 100):
                module.fail_json(
                    msg="If argument rate_type is percentage, the {0} needs to be a value between 0 and 100 inclusive, got {1}".format(
                        rate_name,
                        rate_value,
                    )
                )
        return {
            percentage: "{0:.6f}".format(float(rate)),
            pps: "unspecified",
            burst_percentage: "{0:.6f}".format(float(burst_rate)),
            burst_pps: "unspecified",
        }
    elif rate_type == "pps":
        return {pps: rate, burst_pps: burst_rate}


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        storm_control_policy=dict(
            type="str", required=False, aliases=["name", "storm_control", "storm_control_name"]
        ),  # Not required for querying all objects
        description=dict(type="str", aliases=["descr"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        name_alias=dict(type="str"),
        all_types_configuration=dict(type="dict", options=storm_control_policy_rate_spec(), aliases=["all_types"]),
        broadcast_configuration=dict(type="dict", options=storm_control_policy_rate_spec(), aliases=["broadcast"]),
        multicast_configuration=dict(type="dict", options=storm_control_policy_rate_spec(), aliases=["multicast"]),
        unicast_configuration=dict(type="dict", options=storm_control_policy_rate_spec(), aliases=["unicast"]),
        storm_control_action=dict(type="str", choices=["drop", "shutdown"]),
        storm_control_soak_action=dict(type="int"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["storm_control_policy"]],
            ["state", "present", ["storm_control_policy"]],
        ],
        mutually_exclusive=[
            ("all_types_configuration", "broadcast_configuration"),
            ("all_types_configuration", "multicast_configuration"),
            ("all_types_configuration", "unicast_configuration"),
        ],
    )

    aci = ACIModule(module)

    storm_control_policy = module.params.get("storm_control_policy")
    description = module.params.get("description")
    state = module.params.get("state")
    name_alias = module.params.get("name_alias")
    storm_control_action = module.params.get("storm_control_action")
    storm_control_soak_action = module.params.get("storm_control_soak_action")
    all_types_configuration = module.params.get("all_types_configuration")
    broadcast_configuration = module.params.get("broadcast_configuration")
    multicast_configuration = module.params.get("multicast_configuration")
    unicast_configuration = module.params.get("unicast_configuration")

    rates_input = {}

    if all_types_configuration is not None:
        rates_input.update(get_rates_configuration(module, all_types_configuration, "rate", "ratePps", "burstRate", "burstPps"))
        storm_control_types = "Invalid"
    elif any([broadcast_configuration, multicast_configuration, unicast_configuration]):
        rates_input.update(get_rates_configuration(module, broadcast_configuration, "bcRate", "bcRatePps", "bcBurstRate", "bcBurstPps"))
        rates_input.update(get_rates_configuration(module, multicast_configuration, "mcRate", "mcRatePps", "mcBurstRate", "mcBurstPps"))
        rates_input.update(get_rates_configuration(module, unicast_configuration, "uucRate", "uucRatePps", "uucBurstRate", "uucBurstPps"))
        storm_control_types = "Valid"

    aci.construct_url(
        root_class=dict(
            aci_class="infraInfra",
            aci_rn="infra",
        ),
        subclass_1=dict(
            aci_class="stormctrlIfPol",
            aci_rn="stormctrlifp-{0}".format(storm_control_policy),
            module_object=storm_control_policy,
            target_filter={"name": storm_control_policy},
        ),
    )

    aci.get_existing()

    if state == "present":
        class_config = dict(
            name=storm_control_policy,
            descr=description,
            nameAlias=name_alias,
            isUcMcBcStormPktCfgValid=storm_control_types,
            stormCtrlAction=storm_control_action,
            stormCtrlSoakInstCount=storm_control_soak_action,
        )
        class_config.update(rates_input)
        aci.payload(
            aci_class="stormctrlIfPol",
            class_config=class_config,
        )

        aci.get_diff(aci_class="stormctrlIfPol")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Dag Wieers (@dagwieers) <dag@wieers.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: mso_site
short_description: Manage sites
description:
- Manage sites on Cisco ACI Multi-Site.
author:
- Dag Wieers (@dagwieers)
options:
  apic_password:
    description:
    - The password for the APICs.
    - The apic_password attribute is not supported when using with ND platform.
    - See the ND collection for complete site management.
    type: str
  apic_site_id:
    description:
    - The site ID of the APICs.
    type: str
  apic_username:
    description:
    - The username for the APICs.
    - The apic_username attribute is not supported when using with ND platform.
    - See the ND collection for complete site management.
    type: str
    default: admin
  apic_login_domain:
    description:
    - The AAA login domain for the username for the APICs.
    - The apic_login_domain attribute is not supported when using with ND platform.
    - See the ND collection for complete site management.
    type: str
  site:
    description:
    - The name of the site.
    type: str
    aliases: [ name ]
  labels:
    description:
    - The labels for this site.
    - Labels that do not already exist will be automatically created.
    - The labels attribute is not supported when using with ND platform.
    - See the ND collection for complete site management.
    type: list
    elements: str
  location:
    description:
    - Location of the site.
    - The location attribute is not supported when using with ND platform.
    - See the ND collection for complete site management.
    type: dict
    suboptions:
      latitude:
        description:
        - The latitude of the location of the site.
        type: float
      longitude:
        description:
        - The longitude of the location of the site.
        type: float
  urls:
    description:
    - A list of URLs to reference the APICs.
    - The urls attribute is not supported when using with ND platform.
    - See the ND collection for complete site management.
    type: list
    elements: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Add a new site
  cisco.mso.mso_site:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    site: north_europe
    description: North European Datacenter
    apic_username: mso_admin
    apic_password: AnotherSecretPassword
    apic_site_id: 12
    urls:
      - 10.2.3.4
      - 10.2.4.5
      - 10.3.5.6
    labels:
      - NEDC
      - Europe
      - Diegem
    location:
      latitude: 50.887318
      longitude: 4.447084
    state: present

- name: Remove a site
  cisco.mso.mso_site:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    site: north_europe
    state: absent

- name: Query a site
  cisco.mso.mso_site:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    site: north_europe
    state: query
  register: query_result

- name: Query all sites
  cisco.mso.mso_site:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    state: query
  register: query_result
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec


def main():
    location_arg_spec = dict(
        latitude=dict(type="float"),
        longitude=dict(type="float"),
    )

    argument_spec = mso_argument_spec()
    argument_spec.update(
        apic_password=dict(type="str", no_log=True),
        apic_site_id=dict(type="str"),
        apic_username=dict(type="str", default="admin"),
        apic_login_domain=dict(type="str"),
        labels=dict(type="list", elements="str"),
        location=dict(type="dict", options=location_arg_spec),
        site=dict(type="str", aliases=["name"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        urls=dict(type="list", elements="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["site"]],
            ["state", "present", ["apic_site_id", "site"]],
        ],
    )

    apic_username = module.params.get("apic_username")
    apic_password = module.params.get("apic_password")
    apic_site_id = module.params.get("apic_site_id")
    site = module.params.get("site")
    location = module.params.get("location")
    if location is not None:
        latitude = module.params.get("location")["latitude"]
        longitude = module.params.get("location")["longitude"]
    state = module.params.get("state")
    urls = module.params.get("urls")
    apic_login_domain = module.params.get("apic_login_domain")

    mso = MSOModule(module)

    site_id = None
    path = "sites"
    api_version = "v1"
    if mso.platform == "nd":
        api_version = "v2"

    # Convert labels
    labels = mso.lookup_labels(module.params.get("labels"), "site")

    # Query for mso.existing object(s)
    if site:
        if mso.platform == "nd":
            site_info = mso.get_obj(path, api_version=api_version, common=dict(name=site))
            path = "sites/manage"
            if site_info:
                # If we found an existing object, continue with it
                site_id = site_info.get("id")
                if site_id is not None and site_id != "":
                    # Checking if site is managed by MSO
                    mso.existing = site_info
                    path = "sites/manage/{id}".format(id=site_id)
        else:
            mso.existing = mso.get_obj(path, name=site)
            if mso.existing:
                # If we found an existing object, continue with it
                site_id = mso.existing.get("id")
                path = "sites/{id}".format(id=site_id)

    else:
        mso.existing = mso.query_objs(path, api_version=api_version)

    if state == "query":
        pass

    elif state == "absent":
        mso.previous = mso.existing
        if mso.existing:
            if module.check_mode:
                mso.existing = {}
            else:
                mso.request(path, method="DELETE", qs=dict(force="true"), api_version=api_version)
                mso.existing = {}

    elif state == "present":
        mso.previous = mso.existing

        if mso.platform == "nd":
            if mso.existing:
                payload = mso.existing
            else:
                if site_info:
                    payload = site_info
                    payload["common"]["siteId"] = apic_site_id
                else:
                    mso.fail_json(msg="Site '{0}' is not a valid Site configured at ND-level. Add Site to ND first.".format(site))

        else:
            payload = dict(
                apicSiteId=apic_site_id,
                id=site_id,
                name=site,
                urls=urls,
                labels=labels,
                username=apic_username,
                password=apic_password,
            )

            if location is not None:
                payload["location"] = dict(
                    lat=latitude,
                    long=longitude,
                )

            if apic_login_domain is not None and apic_login_domain not in ["", "local", "Local"]:
                payload["username"] = "apic#{0}\\{1}".format(apic_login_domain, apic_username)

        mso.sanitize(payload, collate=True)

        if mso.existing:
            if mso.check_changed():
                if module.check_mode:
                    mso.existing = mso.proposed
                else:
                    mso.existing = mso.request(path, method="PUT", data=mso.sent, api_version=api_version)
        else:
            if module.check_mode:
                mso.existing = mso.proposed
            else:
                mso.existing = mso.request(path, method="POST", data=mso.sent, api_version=api_version)

    if "password" in mso.existing:
        mso.existing["password"] = "******"

    mso.exit_json()


if __name__ == "__main__":
    main()



# Copyright (c) 2017-2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import abc
import typing as t

from ansible.module_utils.common.validation import (
    check_type_bool,
    check_type_dict,
    check_type_float,
    check_type_int,
    check_type_list,
    check_type_str,
)

if t.TYPE_CHECKING:
    AnsibleType = t.Literal["int", "str", "float", "bool"]  # pragma: no cover


def ensure_type(value: t.Any, type_name: str) -> t.Any:
    if type_name == "str":
        return check_type_str(value)
    if type_name == "list":
        return check_type_list(value)
    if type_name == "dict":
        return check_type_dict(value)
    if type_name == "bool":
        return check_type_bool(value)
    if type_name == "int":
        return check_type_int(value)
    if type_name == "float":
        return check_type_float(value)
    return value


class ProviderInformation(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def get_zone_id_type(self) -> AnsibleType:
        """
        Return the (short) type for zone IDs, like ``'int'`` or ``'str'``.
        """

    @abc.abstractmethod
    def get_record_id_type(self) -> AnsibleType:
        """
        Return the (short) type for record IDs, like ``'int'`` or ``'str'``.
        """

    @abc.abstractmethod
    def get_record_default_ttl(self) -> int | None:
        """
        Return the default TTL for records, like 300, 3600 or None.
        None means that some other TTL (usually from the zone) will be used.
        """

    @abc.abstractmethod
    def get_supported_record_types(self) -> list[str]:
        """
        Return a list of supported record types.
        """

    def normalize_prefix(self, prefix: str | None) -> str | None:
        """
        Given a prefix (string or ``None``), return its normalized form.

        The result should always be None for the trivial prefix, and a non-zero length DNS name
        for a non-trivial prefix.

        If a provider supports other identifiers for the trivial prefix, such as '@', this
        function needs to convert them to ``None`` as well.
        """
        return prefix or None

    def supports_bulk_actions(self) -> bool:
        """
        Return whether the API supports some kind of bulk actions.
        """
        return False

    @abc.abstractmethod
    def txt_record_handling(
        self,
    ) -> t.Literal["decoded", "encoded", "encoded-no-char-encoding"]:
        """
        Return how the API handles TXT records.

        Returns one of the following strings:
        * 'decoded' - the API works with unencoded values
        * 'encoded' - the API works with encoded values
        * 'encoded-no-char-encoding' - the API works with encoded values, but without character encoding
        """

    def txt_character_encoding(self) -> t.Literal["decimal", "octal"]:
        """
        Return how the API handles escape sequences in TXT records.

        Returns one of the following strings:
        * 'octal' - the API works with octal escape sequences
        * 'decimal' - the API works with decimal escape sequences

        This return value is only used if txt_record_handling returns 'encoded'.

        Note: the default return value changed from 'octal' to 'decimal' in community.dns 3.0.0.
        """
        return "decimal"

    def txt_always_quote(self) -> bool:
        """
        Return whether TXT records sent to the API should always be quoted.

        Returns a boolean.

        This return value is only used if txt_record_handling does not return 'decoded'.
        """
        return False



#!/usr/bin/python
#
# Copyright (c) 2018 Zim Kalinowski, <zikalino@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_devtestlabvirtualmachine
version_added: "0.1.2"
short_description: Manage Azure DevTest Lab Virtual Machine instance
description:
    - Create, update and delete instance of Azure DevTest Lab Virtual Machine.

options:
    resource_group:
        description:
            - The name of the resource group.
        required: True
        type: str
    lab_name:
        description:
            - The name of the lab.
        required: True
        type: str
    name:
        description:
            - The name of the virtual machine.
        required: True
        type: str
    notes:
        description:
            - The notes of the virtual machine.
        type: str
    os_type:
        description:
            - Base type of operating system.
        type: str
        choices:
            - windows
            - linux
            - Windows
            - Linux
    vm_size:
        description:
            - A valid Azure VM size value. For example, C(Standard_D4).
            - The list of choices varies depending on the subscription and location. Check your subscription for available choices.
            - Available values can be found on this website, link U(https://docs.microsoft.com/en-us/azure/virtual-machines/windows/sizes-general).
            - Required when I(state=present).
        type: str
    user_name:
        description:
            - The user name of the virtual machine.
        type: str
    password:
        description:
            - The password of the virtual machine administrator.
        type: str
    ssh_key:
        description:
            - The SSH key of the virtual machine administrator.
        type: str
    lab_subnet:
        description:
            - An existing subnet within lab's virtual network.
            - It can be the subnet's resource id.
            - It can be a dict which contains C(virtual_network_name) and C(name).
        type: raw
    disallow_public_ip_address:
        description:
            - Indicates whether the virtual machine is to be created without a public IP address.
        type: str
    artifacts:
        description:
            - The artifacts to be installed on the virtual machine.
        type: list
        elements: dict
        suboptions:
            source_name:
                description:
                    - The artifact's source name.
                type: str
            source_path:
                description:
                    - The artifact's path in the source repository.
                type: str
            parameters:
                description:
                    - The parameters of the artifact.
                type: list
                elements: dict
                suboptions:
                    name:
                        description:
                            - The name of the artifact parameter.
                        type: str
                    value:
                        description:
                            - The value of the artifact parameter.
                        type: str
    image:
        description:
            - The Microsoft Azure Marketplace image reference of the virtual machine.
        type: dict
        suboptions:
            offer:
                description:
                    - The offer of the gallery image.
                type: str
            publisher:
                description:
                    - The publisher of the gallery image.
                type: str
            sku:
                description:
                    - The SKU of the gallery image.
                type: str
            os_type:
                description:
                    - The OS type of the gallery image.
                type: str
            version:
                description:
                    - The version of the gallery image.
                type: str
    expiration_date:
        description:
            - The expiration date for VM.
        type: str
    allow_claim:
        description:
            - Indicates whether another user can take ownership of the virtual machine.
        type: str
    storage_type:
        description:
            - Storage type to use for virtual machine.
        type: str
        choices:
            - standard
            - premium
    is_authentication_with_ssh_key:
        description:
            - Indicates whether this virtual machine uses an SSH key for authentication.
            - I(is_authentication_with_ssh_key=True) when I(ssh_key) is set for ssh authentication.
        type: bool
    state:
        description:
            - Assert the state of the Virtual Machine.
            - Use C(present) to create or update an Virtual Machine and C(absent) to delete it.
        type: str
        default: present
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Zim Kalinowski (@zikalino)

'''

EXAMPLES = '''
- name: Create (or update) Virtual Machine
  azure_rm_devtestlabvirtualmachine:
    resource_group: myrg
    lab_name: mylab
    name: myvm
    notes: Virtual machine notes....
    os_type: linux
    vm_size: Standard_A2_v2
    user_name: vmadmin
    password: "{{ password }}"
    lab_subnet:
      name: myvnSubnet
      virtual_network_name: myvn
    disallow_public_ip_address: false
    image:
      offer: 0001-com-ubuntu-server-focal
      publisher: Canonical
      sku: 20_04-lts
      os_type: Linux
      version: latest
    artifacts:
      - source_name: myartifact
        source_path: "/Artifacts/linux-install-mongodb"
    allow_claim: false
    expiration_date: "2019-02-22T01:49:12.117974Z"
'''

RETURN = '''
id:
    description:
        - The identifier of the DTL Virtual Machine resource.
    returned: always
    type: str
    sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourcegroups/myrg/providers/microsoft.devtestlab/labs/mylab/virtualmachines/myvm
compute_id:
    description:
        - The identifier of the underlying Compute Virtual Machine resource.
    returned: always
    type: str
    sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourcegroups/myrg/providers/microsoft.devtestlab/labs/mylab/virtualmachines/myvm
fqdn:
    description:
        - Fully qualified domain name or IP Address of the virtual machine.
    returned: always
    type: str
    sample: myvm.eastus.cloudapp.azure.com
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
from ansible.module_utils.common.dict_transformations import _snake_to_camel

try:
    from azure.core.polling import LROPoller
    from azure.core.exceptions import ResourceNotFoundError
    from azure.mgmt.devtestlabs import DevTestLabsClient
except ImportError:
    # This is handled in azure_rm_common
    pass


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMVirtualMachine(AzureRMModuleBase):
    """Configuration class for an Azure RM Virtual Machine resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            lab_name=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            notes=dict(
                type='str'
            ),
            os_type=dict(
                type='str',
                choices=['linux', 'windows', 'Linux', 'Windows']
            ),
            vm_size=dict(
                type='str'
            ),
            user_name=dict(
                type='str'
            ),
            password=dict(
                type='str',
                no_log=True
            ),
            ssh_key=dict(
                type='str',
                no_log=True
            ),
            lab_subnet=dict(
                type='raw'
            ),
            disallow_public_ip_address=dict(
                type='str'
            ),
            artifacts=dict(
                type='list',
                elements='dict',
                options=dict(
                    source_name=dict(
                        type='str',
                    ),
                    source_path=dict(
                        type='str'
                    ),
                    parameters=dict(
                        type='list',
                        elements='dict',
                        options=dict(
                            name=dict(
                                type='str'
                            ),
                            value=dict(
                                type='str'
                            )
                        )
                    )
                )
            ),
            image=dict(
                type='dict',
                options=dict(
                    offer=dict(
                        type='str'
                    ),
                    publisher=dict(
                        type='str'
                    ),
                    sku=dict(
                        type='str'
                    ),
                    os_type=dict(
                        type='str'
                    ),
                    version=dict(
                        type='str'
                    )
                )
            ),
            expiration_date=dict(
                type='str'
            ),
            allow_claim=dict(
                type='str'
            ),
            storage_type=dict(
                type='str',
                choices=['standard', 'premium']
            ),
            is_authentication_with_ssh_key=dict(
                type='bool'
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        required_if = [
            ('state', 'present', ['image', 'lab_subnet', 'vm_size', 'os_type']),
            ('is_authentication_with_ssh_key', True, ['ssh_key'])
        ]

        self.resource_group = None
        self.lab_name = None
        self.name = None
        self.lab_virtual_machine = dict()

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.to_do = Actions.NoAction

        super(AzureRMVirtualMachine, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                    supports_check_mode=True,
                                                    supports_tags=True,
                                                    required_if=required_if)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                self.lab_virtual_machine[key] = kwargs[key]

        self.lab_virtual_machine['gallery_image_reference'] = self.lab_virtual_machine.pop('image', None)

        if self.lab_virtual_machine.get('artifacts') is not None:
            for artifact in self.lab_virtual_machine.get('artifacts'):
                source_name = artifact.pop('source_name')
                source_path = artifact.pop('source_path')
                template = "/subscriptions/{0}/resourceGroups/{1}/providers/Microsoft.DevTestLab/labs/{2}/artifactsources/{3}{4}"
                artifact['artifact_id'] = template.format(self.subscription_id, self.resource_group, self.lab_name, source_name, source_path)

        self.lab_virtual_machine['size'] = self.lab_virtual_machine.pop('vm_size') if self.lab_virtual_machine.get('vm_size') else None
        self.lab_virtual_machine['os_type'] = _snake_to_camel(self.lab_virtual_machine['os_type']) if self.lab_virtual_machine.get('os_type') else None

        if self.lab_virtual_machine.get('storage_type'):
            self.lab_virtual_machine['storage_type'] = _snake_to_camel(self.lab_virtual_machine['storage_type'], True)

        lab_subnet = self.lab_virtual_machine.pop('lab_subnet') if self.lab_virtual_machine.get('lab_subnet') else dict()

        if isinstance(lab_subnet, str):
            vn_and_subnet = lab_subnet.split('/subnets/')
            if (len(vn_and_subnet) == 2):
                self.lab_virtual_machine['lab_virtual_network_id'] = vn_and_subnet[0]
                self.lab_virtual_machine['lab_subnet_name'] = vn_and_subnet[1]
            else:
                self.fail("Invalid 'lab_subnet' resource id format")
        else:
            template = "/subscriptions/{0}/resourceGroups/{1}/providers/Microsoft.DevTestLab/labs/{2}/virtualnetworks/{3}"
            self.lab_virtual_machine['lab_virtual_network_id'] = template.format(self.subscription_id,
                                                                                 self.resource_group,
                                                                                 self.lab_name,
                                                                                 lab_subnet.get('virtual_network_name'))
            self.lab_virtual_machine['lab_subnet_name'] = lab_subnet.get('name')

        response = None

        self.mgmt_client = self.get_mgmt_svc_client(DevTestLabsClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        old_response = self.get_virtualmachine()

        if not old_response:
            self.log("Virtual Machine instance doesn't exist")
            if self.state == 'absent':
                self.log("Old instance didn't exist")
            else:
                self.to_do = Actions.Create
            # get location from the lab as it has to be the same and has to be specified (why??)
            lab = self.get_devtestlab()
            self.lab_virtual_machine['location'] = lab['location']
        else:
            self.log("Virtual Machine instance already exists")
            if self.state == 'absent':
                self.to_do = Actions.Delete
            elif self.state == 'present':
                self.lab_virtual_machine['location'] = old_response['location']

                if old_response['size'].lower() != self.lab_virtual_machine.get('size').lower():
                    self.lab_virtual_machine['size'] = old_response['size']
                    self.module.warn("Property 'size' cannot be changed")

                if self.lab_virtual_machine.get('storage_type') is not None and \
                   old_response['storage_type'].lower() != self.lab_virtual_machine.get('storage_type').lower():
                    self.lab_virtual_machine['storage_type'] = old_response['storage_type']
                    self.module.warn("Property 'storage_type' cannot be changed")

                if old_response.get('gallery_image_reference', {}) != self.lab_virtual_machine.get('gallery_image_reference', {}):
                    self.lab_virtual_machine['gallery_image_reference'] = old_response['gallery_image_reference']
                    self.module.warn("Property 'image' cannot be changed")

                # currently artifacts can be only specified when vm is created
                # and in addition we don't have detailed information, just a number of "total artifacts"
                if len(self.lab_virtual_machine.get('artifacts', [])) != old_response['artifact_deployment_status']['total_artifacts']:
                    self.module.warn("Property 'artifacts' cannot be changed")

                if self.lab_virtual_machine.get('is_authentication_with_ssh_key') is not None:
                    if bool(old_response['is_authentication_with_ssh_key']) != bool(self.lab_virtual_machine['is_authentication_with_ssh_key']):
                        self.module.warn("Property 'is_authentication_with_ssh_key' cannot be changed")
                else:
                    self.lab_virtual_machine['is_authentication_with_ssh_key'] = old_response['is_authentication_with_ssh_key']

                if self.lab_virtual_machine.get('disallow_public_ip_address') is not None:
                    if old_response['disallow_public_ip_address'] != self.lab_virtual_machine.get('disallow_public_ip_address'):
                        self.module.warn("Property 'disallow_public_ip_address' cannot be changed")
                self.lab_virtual_machine['disallow_public_ip_address'] = old_response['disallow_public_ip_address']

                if self.lab_virtual_machine.get('allow_claim') is not None:
                    if old_response['allow_claim'] != self.lab_virtual_machine.get('allow_claim'):
                        self.module.warn("Property 'allow_claim' cannot be changed")
                self.lab_virtual_machine['allow_claim'] = old_response['allow_claim']

                if self.lab_virtual_machine.get('notes') is not None:
                    if old_response['notes'] != self.lab_virtual_machine.get('notes'):
                        self.to_do = Actions.Update
                else:
                    self.lab_virtual_machine['notes'] = old_response['notes']

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.log("Need to Create / Update the Virtual Machine instance")

            self.results['changed'] = True
            if self.check_mode:
                return self.results

            response = self.create_update_virtualmachine()

            self.log("Creation / Update done")
        elif self.to_do == Actions.Delete:
            self.log("Virtual Machine instance deleted")
            self.results['changed'] = True

            if self.check_mode:
                return self.results

            self.delete_virtualmachine()
        else:
            self.log("Virtual Machine instance unchanged")
            self.results['changed'] = False
            response = old_response

        if self.state == 'present':
            self.results.update({
                'id': response.get('id', None),
                'compute_id': response.get('compute_id', None),
                'fqdn': response.get('fqdn', None)
            })
        return self.results

    def create_update_virtualmachine(self):
        '''
        Creates or updates Virtual Machine with the specified configuration.

        :return: deserialized Virtual Machine instance state dictionary
        '''
        self.log("Creating / Updating the Virtual Machine instance {0}".format(self.name))

        try:
            response = self.mgmt_client.virtual_machines.begin_create_or_update(resource_group_name=self.resource_group,
                                                                                lab_name=self.lab_name,
                                                                                name=self.name,
                                                                                lab_virtual_machine=self.lab_virtual_machine)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)

        except Exception as exc:
            self.log('Error attempting to create the Virtual Machine instance.')
            self.fail("Error creating the Virtual Machine instance: {0}".format(str(exc)))
        return response.as_dict()

    def delete_virtualmachine(self):
        '''
        Deletes specified Virtual Machine instance in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the Virtual Machine instance {0}".format(self.name))
        try:
            response = self.mgmt_client.virtual_machines.begin_delete(resource_group_name=self.resource_group,
                                                                      lab_name=self.lab_name,
                                                                      name=self.name)
        except Exception as e:
            self.log('Error attempting to delete the Virtual Machine instance.')
            self.fail("Error deleting the Virtual Machine instance: {0}".format(str(e)))

        if isinstance(response, LROPoller):
            response = self.get_poller_result(response)

        return True

    def get_virtualmachine(self):
        '''
        Gets the properties of the specified Virtual Machine.

        :return: deserialized Virtual Machine instance state dictionary
        '''
        self.log("Checking if the Virtual Machine instance {0} is present".format(self.name))
        found = False
        try:
            response = self.mgmt_client.virtual_machines.get(resource_group_name=self.resource_group,
                                                             lab_name=self.lab_name,
                                                             name=self.name)
            found = True
            self.log("Response : {0}".format(response))
            self.log("Virtual Machine instance : {0} found".format(response.name))
        except ResourceNotFoundError as e:
            self.log('Did not find the Virtual Machine instance.')
        if found is True:
            return response.as_dict()

        return False

    def get_devtestlab(self):
        '''
        Gets the properties of the specified DevTest Lab.

        :return: deserialized DevTest Lab instance state dictionary
        '''
        self.log("Checking if the DevTest Lab instance {0} is present".format(self.lab_name))
        try:
            response = self.mgmt_client.labs.get(resource_group_name=self.resource_group,
                                                 name=self.lab_name)
            self.log("Response : {0}".format(response))
            self.log("DevTest Lab instance : {0} found".format(response.name))
            return response.as_dict()
        except ResourceNotFoundError as e:
            self.fail('Did not find the DevTest Lab instance.')
            return False


def main():
    """Main execution"""
    AzureRMVirtualMachine()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the
# ansible.content_builder.
#
# Manually editing this file is not advised.
#
# To update the argspec make the desired changes
# in the module docstring and re-run
# ansible.content_builder.
#
#############################################

"""
The arg spec for the nxos_bgp_templates module
"""


class Bgp_templatesArgs(object):  # pylint: disable=R0903
    """The arg spec for the nxos_bgp_templates module"""

    argument_spec = {
        "running_config": {"type": "str"},
        "config": {
            "type": "dict",
            "options": {
                "as_number": {"type": "str"},
                "neighbor": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "name": {"type": "str"},
                        "address_family": {
                            "type": "list",
                            "elements": "dict",
                            "options": {
                                "afi": {
                                    "type": "str",
                                    "choices": [
                                        "ipv4",
                                        "ipv6",
                                        "link-state",
                                        "l2vpn",
                                    ],
                                    "required": True,
                                },
                                "safi": {
                                    "type": "str",
                                    "choices": ["unicast", "multicast", "mvpn", "evpn"],
                                },
                                "advertise_map": {
                                    "type": "dict",
                                    "options": {
                                        "route_map": {
                                            "type": "str",
                                            "required": True,
                                        },
                                        "exist_map": {"type": "str"},
                                        "non_exist_map": {"type": "str"},
                                    },
                                },
                                "advertisement_interval": {"type": "int"},
                                "allowas_in": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "max_occurences": {"type": "int"},
                                    },
                                },
                                "as_override": {"type": "bool"},
                                "capability": {
                                    "type": "dict",
                                    "options": {
                                        "additional_paths": {
                                            "type": "dict",
                                            "options": {
                                                "receive": {
                                                    "type": "str",
                                                    "choices": [
                                                        "enable",
                                                        "disable",
                                                    ],
                                                },
                                                "send": {
                                                    "type": "str",
                                                    "choices": [
                                                        "enable",
                                                        "disable",
                                                    ],
                                                },
                                            },
                                        },
                                    },
                                },
                                "default_originate": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "route_map": {"type": "str"},
                                    },
                                },
                                "disable_peer_as_check": {"type": "bool"},
                                "filter_list": {
                                    "type": "dict",
                                    "options": {
                                        "inbound": {"type": "str"},
                                        "outbound": {"type": "str"},
                                    },
                                },
                                "inherit": {
                                    "type": "dict",
                                    "options": {"peer_policy": {"type": "str"}},
                                },
                                "maximum_prefix": {
                                    "type": "dict",
                                    "options": {
                                        "max_prefix_limit": {"type": "int"},
                                        "generate_warning_threshold": {
                                            "type": "int",
                                        },
                                        "restart_interval": {"type": "int"},
                                        "warning_only": {"type": "bool"},
                                    },
                                },
                                "next_hop_self": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "all_routes": {"type": "bool"},
                                    },
                                },
                                "next_hop_third_party": {"type": "bool"},
                                "prefix_list": {
                                    "type": "dict",
                                    "options": {
                                        "inbound": {"type": "str"},
                                        "outbound": {"type": "str"},
                                    },
                                },
                                "route_map": {
                                    "type": "dict",
                                    "options": {
                                        "inbound": {"type": "str"},
                                        "outbound": {"type": "str"},
                                    },
                                },
                                "route_reflector_client": {"type": "bool"},
                                "send_community": {
                                    "type": "str",
                                    "choices": ["standard", "extended", "both"],
                                },
                                "soft_reconfiguration_inbound": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "always": {"type": "bool"},
                                    },
                                },
                                "soo": {"type": "str"},
                                "suppress_inactive": {"type": "bool"},
                                "unsuppress_map": {"type": "str"},
                                "weight": {"type": "int"},
                            },
                        },
                        "bfd": {
                            "type": "dict",
                            "options": {
                                "set": {"type": "bool"},
                                "singlehop": {"type": "bool"},
                                "multihop": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "interval": {
                                            "type": "dict",
                                            "options": {
                                                "tx_interval": {"type": "int", "required": True},
                                                "min_rx_interval": {
                                                    "type": "int",
                                                    "required": True,
                                                },
                                                "multiplier": {"type": "int", "required": True},
                                            },
                                        },
                                    },
                                },
                            },
                        },
                        "bmp_activate_server": {"type": "int"},
                        "capability": {
                            "type": "dict",
                            "options": {"suppress_4_byte_as": {"type": "bool"}},
                        },
                        "description": {"type": "str"},
                        "disable_connected_check": {"type": "bool"},
                        "dont_capability_negotiate": {"type": "bool"},
                        "dscp": {"type": "str"},
                        "dynamic_capability": {"type": "bool"},
                        "ebgp_multihop": {"type": "int"},
                        "graceful_shutdown": {
                            "type": "dict",
                            "options": {
                                "activate": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "route_map": {"type": "str"},
                                    },
                                },
                            },
                        },
                        "inherit": {
                            "type": "dict",
                            "options": {"peer_session": {"type": "str"}},
                        },
                        "local_as": {"type": "str"},
                        "log_neighbor_changes": {
                            "type": "dict",
                            "options": {
                                "set": {"type": "bool"},
                                "disable": {"type": "bool"},
                            },
                        },
                        "low_memory": {
                            "type": "dict",
                            "options": {"exempt": {"type": "bool"}},
                        },
                        "password": {
                            "type": "dict",
                            "no_log": False,
                            "options": {
                                "encryption": {"type": "int"},
                                "key": {"type": "str", "no_log": True},
                            },
                        },
                        "path_attribute": {
                            "type": "list",
                            "elements": "dict",
                            "options": {
                                "action": {
                                    "type": "str",
                                    "choices": ["discard", "treat-as-withdraw"],
                                },
                                "type": {"type": "int"},
                                "range": {
                                    "type": "dict",
                                    "options": {
                                        "start": {"type": "int"},
                                        "end": {"type": "int"},
                                    },
                                },
                            },
                        },
                        "remote_as": {"type": "str"},
                        "remove_private_as": {
                            "type": "dict",
                            "options": {
                                "set": {"type": "bool"},
                                "replace_as": {"type": "bool"},
                                "all": {"type": "bool"},
                            },
                        },
                        "shutdown": {"type": "bool"},
                        "timers": {
                            "type": "dict",
                            "options": {
                                "keepalive": {"type": "int"},
                                "holdtime": {"type": "int"},
                            },
                        },
                        "transport": {
                            "type": "dict",
                            "options": {
                                "connection_mode": {
                                    "type": "dict",
                                    "options": {"passive": {"type": "bool"}},
                                },
                            },
                        },
                        "ttl_security": {
                            "type": "dict",
                            "options": {"hops": {"type": "int"}},
                        },
                        "update_source": {"type": "str"},
                    },
                },
            },
        },
        "state": {
            "type": "str",
            "choices": [
                "merged",
                "replaced",
                "overridden",
                "deleted",
                "parsed",
                "gathered",
                "rendered",
            ],
            "default": "merged",
        },
    }  # pylint: disable=C0301



# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

"""
pyats parser

This is the pyats parser for use with the cli_parse module and action plugin.
https://developer.cisco.com/docs/pyats/#!parsing-device-output

"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
    author: Bradley Thornton (@cidrblock)
    name: pyats
    short_description: Define configurable options for C(pyats) sub-plugin of C(cli_parse) module
    description:
    - This plugin documentation provides the configurable options that can be passed
      to the I(ansible.utils.cli_parse) plugins when I(ansible.netcommon.pyats) is used as a value for
      I(name) option.
    version_added: 1.0.0
"""

EXAMPLES = r"""
- name: "Run command and parse with pyats"
  ansible.utils.cli_parse:
    command: "show interface"
    parser:
      name: ansible.netcommon.pyats
  register: nxos_pyats_command

- name: "Pass text and command"
  ansible.utils.cli_parse:
    text: "{{ nxos_pyats_command['stdout'] }}"
    parser:
      name: ansible.netcommon.pyats
      command: show interface
  register: nxos_pyats_text
"""

from ansible.module_utils.basic import missing_required_lib
from ansible.module_utils.common.text.converters import to_native
from ansible_collections.ansible.utils.plugins.plugin_utils.base.cli_parser import CliParserBase


try:
    from genie.conf.base import Device

    HAS_GENIE = True
except ImportError:
    HAS_GENIE = False

try:
    from pyats.datastructures import AttrDict

    HAS_PYATS = True
except ImportError:
    HAS_PYATS = False


class CliParser(CliParserBase):
    """The pyats parser class
    Convert raw text to structured data using pyats/genie
    """

    DEFAULT_TEMPLATE_EXTENSION = None
    PROVIDE_TEMPLATE_CONTENTS = False

    @staticmethod
    def _check_reqs():
        """Check the prerequisites are installed for pyats/genie

        :return dict: A dict with a list of errors
        """
        errors = []
        if not HAS_GENIE:
            errors.append(missing_required_lib("genie"))
        if not HAS_PYATS:
            errors.append(missing_required_lib("pyats"))
        return errors

    def _check_vars(self):
        """Ensure specific args are set

        :return: A dict with a list of errors
        :rtype: dict
        """
        errors = []
        if not self._task_args.get("parser").get("command"):
            errors.append("The pyats parser requires parser/command be provided.")
        return errors

    def _transform_ansible_network_os(self):
        """Transform the ansible_network_os to a pyats OS
        The last part of the fully qualified name is used
        org.name.platform => platform

        In the case of ios, the os is assumed to be iosxe
        """
        ane = self._task_vars.get("ansible_network_os", "").split(".")[-1]
        if ane == "ios":
            self._debug("ansible_network_os was ios, using iosxe.")
            ane = "iosxe"
        self._debug("OS set to '{ane}' using 'ansible_network_os'.".format(ane=ane))
        return ane

    def parse(self, *_args, **_kwargs):
        """Std entry point for a cli_parse parse execution

        :return: Errors or parsed text as structured data
        :rtype: dict

        :example:

        The parse function of a parser should return a dict:
        {"errors": [a list of errors]}
        or
        {"parsed": obj}
        """
        errors = self._check_reqs()
        errors.extend(self._check_vars())
        if errors:
            return {"errors": errors}

        command = self._task_args.get("parser").get("command")
        network_os = self._task_args.get("parser").get("os") or self._transform_ansible_network_os()
        cli_output = self._task_args.get("text")

        device = Device("new_device", os=network_os)
        device.custom.setdefault("abstraction", {})["order"] = ["os"]
        device.cli = AttrDict({"execute": None})

        try:
            parsed = device.parse(command, output=cli_output)
        except Exception as exc:
            msg = "The pyats library return an error for '{cmd}' for '{os}'. Error: {err}."
            return {"errors": [msg.format(cmd=command, os=network_os, err=to_native(exc))]}
        return {"parsed": parsed}



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2021, Shreyas Srish (@shrsr) <ssrish@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: mso_schema_template_deploy_status
short_description: Check query of objects before deployment to site
description:
- Check query of objects in a template of a schema
author:
- Shreyas Srish (@shrsr)
options:
  schema:
    description:
    - The name of the schema.
    type: str
    aliases: [ name ]
  template:
    description:
    - The name of the template.
    type: str
  site:
    description:
    - The name of the site.
    type: str
  state:
    description:
    - Use C(query) for listing query of objects.
    type: str
    choices: [ query ]
    default: query
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""

- name: Query status of objects in a template
  cisco.mso.mso_schema_template_deploy_status:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    state: query
  register: query_result

- name: Query status of objects using site
  cisco.mso.mso_schema_template_deploy_status:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    site: ansible_test
    state: query
  register: query_result

- name: Query status of objects in a template associated with a site
  cisco.mso.mso_schema_template_deploy_status:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    site: ansible_test
    state: query
  register: query_result

- name: Query status of objects in all templates
  cisco.mso.mso_schema_template_deploy_status:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    state: query
  register: query_result
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        schema=dict(type="str", aliases=["name"]),
        template=dict(type="str"),
        site=dict(type="str"),
        state=dict(type="str", default="query", choices=["query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "query", ["schema"]],
        ],
    )

    schema = module.params.get("schema")
    template = module.params.get("template")
    if template is not None:
        template = template.replace(" ", "")
    site = module.params.get("site")
    state = module.params.get("state")

    mso = MSOModule(module)

    schema_id = None
    path = "schemas"

    get_schema = mso.get_obj(path, displayName=schema)
    if get_schema:
        schema_id = get_schema.get("id")
        path = "schemas/{id}/policy-states".format(id=schema_id)
    else:
        mso.fail_json(msg="Schema '{0}' not found.".format(schema))

    if state == "query":
        get_data = mso.request(path, method="GET")
        mso.existing = []
        if template:
            for configuration_objects in get_data.get("policyStates"):
                if configuration_objects.get("templateName") == template:
                    mso.existing.append(configuration_objects)
            if not mso.existing:
                mso.fail_json(msg="Template '{0}' not found.".format(template))

        if site:
            mso.existing.clear()
            for configuration_objects in get_data.get("policyStates"):
                if configuration_objects.get("siteId") == mso.lookup_site(site):
                    if template:
                        if configuration_objects.get("templateName") == template:
                            mso.existing = configuration_objects
                    else:
                        mso.existing.append(configuration_objects)
            if template is not None and not mso.existing:
                mso.fail_json(msg="Provided Template '{0}' not associated with Site '{1}'.".format(template, site))

        if template is None and site is None:
            mso.existing = get_data

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) Seth Edwards, 2014
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: librato_annotation
short_description: Create an annotation in Librato
description:
  - Create an annotation event on the given annotation stream O(name). If the annotation stream does not exist, it creates
    one automatically.
author: "Seth Edwards (@Sedward)"
requirements: []
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  user:
    type: str
    description:
      - Librato account username.
    required: true
  api_key:
    type: str
    description:
      - Librato account API key.
    required: true
  name:
    type: str
    description:
      - The annotation stream name.
      - If the annotation stream does not exist, it creates one automatically.
  title:
    type: str
    description:
      - The title of an annotation is a string and may contain spaces.
      - The title should be a short, high-level summary of the annotation for example V(v45 Deployment).
    required: true
  source:
    type: str
    description:
      - A string which describes the originating source of an annotation when that annotation is tracked across multiple members
        of a population.
  description:
    type: str
    description:
      - The description contains extra metadata about a particular annotation.
      - The description should contain specifics on the individual annotation for example V(Deployed 9b562b2 shipped new feature
        foo!).
  start_time:
    type: int
    description:
      - The unix timestamp indicating the time at which the event referenced by this annotation started.
  end_time:
    type: int
    description:
      - The unix timestamp indicating the time at which the event referenced by this annotation ended.
      - For events that have a duration, this is a useful way to annotate the duration of the event.
  links:
    type: list
    elements: dict
    description:
      - See examples.
"""

EXAMPLES = r"""
- name: Create a simple annotation event with a source
  community.general.librato_annotation:
    user: user@example.com
    api_key: XXXXXXXXXXXXXXXXX
    title: App Config Change
    source: foo.bar
    description: This is a detailed description of the config change

- name: Create an annotation that includes a link
  community.general.librato_annotation:
    user: user@example.com
    api_key: XXXXXXXXXXXXXXXXXX
    name: code.deploy
    title: app code deploy
    description: this is a detailed description of a deployment
    links:
      - rel: example
        href: http://www.example.com/deploy

- name: Create an annotation with a start_time and end_time
  community.general.librato_annotation:
    user: user@example.com
    api_key: XXXXXXXXXXXXXXXXXX
    name: maintenance
    title: Maintenance window
    description: This is a detailed description of maintenance
    start_time: 1395940006
    end_time: 1395954406
"""

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url


def post_annotation(module):
    user = module.params["user"]
    api_key = module.params["api_key"]
    name = module.params["name"]
    title = module.params["title"]

    url = f"https://metrics-api.librato.com/v1/annotations/{name}"
    params = {}
    params["title"] = title

    if module.params["source"] is not None:
        params["source"] = module.params["source"]
    if module.params["description"] is not None:
        params["description"] = module.params["description"]
    if module.params["start_time"] is not None:
        params["start_time"] = module.params["start_time"]
    if module.params["end_time"] is not None:
        params["end_time"] = module.params["end_time"]
    if module.params["links"] is not None:
        params["links"] = module.params["links"]

    json_body = module.jsonify(params)

    headers = {}
    headers["Content-Type"] = "application/json"

    # Hack send parameters the way fetch_url wants them
    module.params["url_username"] = user
    module.params["url_password"] = api_key
    response, info = fetch_url(module, url, data=json_body, headers=headers)
    response_code = str(info["status"])
    response_body = info["body"]
    if info["status"] != 201:
        if info["status"] >= 400:
            module.fail_json(msg=f"Request Failed. Response code: {response_code} Response body: {response_body}")
        else:
            module.fail_json(msg=f"Request Failed. Response code: {response_code}")
    response = response.read()
    module.exit_json(changed=True, annotation=response)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            user=dict(required=True),
            api_key=dict(required=True, no_log=True),
            name=dict(),
            title=dict(required=True),
            source=dict(),
            description=dict(),
            start_time=dict(type="int"),
            end_time=dict(type="int"),
            links=dict(type="list", elements="dict"),
        )
    )

    post_annotation(module)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

"""
The keep_keys plugin code
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

import re

from ansible.errors import AnsibleFilterError


def _raise_error(msg):
    """Raise an error message, prepend with filter name
    :param msg: The message
    :type msg: str
    :raises: AnsibleError
    """
    error = "Error when using plugin 'keep_keys': {msg}".format(msg=msg)
    raise AnsibleFilterError(error)


def keep_keys_from_dict_n_list(data, target, matching_parameter):
    if isinstance(data, list):
        list_data = [keep_keys_from_dict_n_list(each, target, matching_parameter) for each in data]
        return [r for r in list_data if r not in ([], {}, None)]
    if isinstance(data, dict):
        keep = {}
        for k, val in data.items():
            key_matches = any(
                (
                    k == t
                    if matching_parameter == "equality"
                    else (
                        (re.match(t, k) is not None)
                        if matching_parameter == "regex"
                        else (
                            k.startswith(t)
                            if matching_parameter == "starts_with"
                            else k.endswith(t)
                        )
                    )
                )
                for t in target
            )
            if key_matches:
                keep[k] = val
            elif isinstance(val, (list, dict)):
                nested_keep = keep_keys_from_dict_n_list(val, target, matching_parameter)
                if nested_keep not in ([], {}, None):
                    keep[k] = nested_keep
        return keep
    return None


def keep_keys(data, target, matching_parameter="equality"):
    """keep selective keys recursively from a given data"
    :param data: The data passed in (data|keep_keys(...))
    :type data: raw
    :param target: List of keys on with operation is to be performed
    :type data: list
    :type elements: string
    :param matching_parameter: matching type of the target keys with data keys
    :type data: str
    """
    if not isinstance(data, (list, dict)):
        _raise_error("Input is not valid for keep operation")
    data = keep_keys_from_dict_n_list(data, target, matching_parameter)
    return data



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networkfirewall_info
short_description: describe AWS Network Firewall firewalls
version_added: 4.0.0
description:
  - A module for describing AWS Network Firewall firewalls.
options:
  arn:
    description:
      - The ARN of the Network Firewall.
      - Mutually exclusive with I(name) and I(vpc_ids).
    required: false
    type: str
  name:
    description:
      - The name of the Network Firewall.
      - Mutually exclusive with I(arn) and I(vpc_ids).
    required: false
    type: str
  vpc_ids:
    description:
      - A List of VPCs to retrieve the firewalls for.
      - Mutually exclusive with I(name) and I(arn).
    required: false
    type: list
    elements: str
    aliases: ['vpcs', 'vpc_id']

author:
  - Mark Chappell (@tremble)
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""

# Describe all firewalls in an account
- community.aws.networkfirewall_info: {}

# Describe a firewall by ARN
- community.aws.networkfirewall_info:
    arn: arn:aws:network-firewall:us-east-1:123456789012:firewall/ExampleFirewall

# Describe a firewall by name
- community.aws.networkfirewall_info:
    name: ExampleFirewall
"""

RETURN = r"""
firewall_list:
  description: A list of ARNs of the matching firewalls.
  type: list
  elements: str
  returned: When a firewall name isn't specified
  example: ['arn:aws:network-firewall:us-east-1:123456789012:firewall/Example1',
            'arn:aws:network-firewall:us-east-1:123456789012:firewall/Example2']

firewalls:
  description: The details of the firewalls
  returned: success
  type: list
  elements: dict
  contains:
    firewall:
      description: The details of the firewall
      type: dict
      returned: success
      contains:
        delete_protection:
          description: A flag indicating whether it is possible to delete the firewall.
          type: str
          returned: success
          example: true
        description:
          description: A description of the firewall.
          type: str
          returned: success
          example: "Description"
        firewall_arn:
          description: The ARN of the firewall.
          type: str
          returned: success
          example: "arn:aws:network-firewall:us-east-1:123456789012:firewall/ExampleFirewall"
        firewall_id:
          description: A unique ID for the firewall.
          type: str
          returned: success
          example: "12345678-abcd-1234-abcd-123456789abc"
        firewall_name:
          description: The name of the firewall.
          type: str
          returned: success
          example: "ExampleFirewall"
        firewall_policy_arn:
          description:  The ARN of the firewall policy used by the firewall.
          type: str
          returned: success
          example: "arn:aws:network-firewall:us-east-1:123456789012:firewall-policy/ExamplePolicy"
        firewall_policy_change_protection:
          description:
            - A flag indicating whether it is possible to change which firewall
              policy is used by the firewall.
          type: bool
          returned: success
          example: false
        subnet_change_protection:
          description:
            - A flag indicating whether it is possible to change which subnets
              the firewall endpoints are in.
          type: bool
          returned: success
          example: true
        subnet_mappings:
          description: A list of the subnets the firewall endpoints are in.
          type: list
          elements: dict
          contains:
            subnet_id:
              description: The ID of the subnet.
              type: str
              returned: success
              example: "subnet-12345678"
        tags:
          description: The tags associated with the firewall.
          type: dict
          returned: success
          example: '{"SomeTag": "SomeValue"}'
        vpc_id:
          description: The ID of the VPC that the firewall is used by.
          type: str
          returned: success
          example: "vpc-0123456789abcdef0"
    firewall_metadata:
      description: Metadata about the firewall
      type: dict
      returned: success
      contains:
        configuration_sync_state_summary:
          description:
            - A short summary of the synchronization status of the
              policy and rule groups.
          type: str
          returned: success
          example: "IN_SYNC"
        status:
          description:
            - A short summary of the status of the firewall endpoints.
          type: str
          returned: success
          example: "READY"
        sync_states:
          description:
            - A description, broken down by availability zone, of the status
              of the firewall endpoints as well as the synchronization status
              of the policies and rule groups.
          type: dict
          returned: success
          example:
            {
              "us-east-1a": {
                "attachment": {
                  "endpoint_id": "vpce-123456789abcdef01",
                  "status": "READY",
                  "subnet_id": "subnet-12345678"
                },
                "config": {
                  "arn:aws:network-firewall:us-east-1:123456789012:firewall-policy/Ansible-Example": {
                    "sync_status": "IN_SYNC",
                    "update_token": "abcdef01-0000-0000-0000-123456789abc"
                  },
                  "arn:aws:network-firewall:us-east-1:123456789012:stateful-rulegroup/ExampleDomainList": {
                    "sync_status": "IN_SYNC",
                    "update_token": "12345678-0000-0000-0000-abcdef012345"
                  }
                }
              }
            }
"""

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule
from ansible_collections.community.aws.plugins.module_utils.networkfirewall import NetworkFirewallManager


def main():
    argument_spec = dict(
        name=dict(type="str", required=False),
        arn=dict(type="str", required=False),
        vpc_ids=dict(type="list", required=False, elements="str", aliases=["vpcs", "vpc_id"]),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[
            ["arn", "name", "vpc_ids"],
        ],
    )

    arn = module.params.get("arn")
    name = module.params.get("name")
    vpcs = module.params.get("vpc_ids")

    manager = NetworkFirewallManager(module)

    results = dict(changed=False)

    if name or arn:
        firewall = manager.get_firewall(name=name, arn=arn)
        if firewall:
            results["firewalls"] = [firewall]
        else:
            results["firewalls"] = []
    else:
        if vpcs:
            firewall_list = manager.list(vpc_ids=vpcs)
        else:
            firewall_list = manager.list()
        results["firewall_list"] = firewall_list
        firewalls = [manager.get_firewall(arn=f) for f in firewall_list]
        results["firewalls"] = firewalls

    module.exit_json(**results)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2016, 2017 Jasper Lievisse Adriaanse <j@jasper.la>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: imgadm
short_description: Manage SmartOS images
description:
  - Manage SmartOS virtual machine images through imgadm(8).
author: Jasper Lievisse Adriaanse (@jasperla)
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  force:
    type: bool
    description:
      - Force a given operation (where supported by imgadm(8)).
  pool:
    default: zones
    description:
      - The zpool to import to or delete images from.
    type: str
  source:
    description:
      - URI for the image source.
    type: str
  state:
    required: true
    choices: [present, absent, deleted, imported, updated, vacuumed]
    description:
      - State the object operated on should be in. V(imported) is an alias for for V(present) and V(deleted) for V(absent).
        When set to V(vacuumed) and O(uuid=*), it removes all unused images.
    type: str

  type:
    choices: [imgapi, docker, dsapi]
    default: imgapi
    description:
      - Type for image sources.
    type: str

  uuid:
    description:
      - Image UUID. Can either be a full UUID or V(*) for all images.
    type: str
seealso:
  - name: imgadm(8)
    description: Complete manual page for the command C(imgadm).
    link: https://smartos.org/man/8/imgadm
"""

EXAMPLES = r"""
- name: Import an image
  community.general.imgadm:
    uuid: '70e3ae72-96b6-11e6-9056-9737fd4d0764'
    state: imported

- name: Delete an image
  community.general.imgadm:
    uuid: '70e3ae72-96b6-11e6-9056-9737fd4d0764'
    state: deleted

- name: Update all images
  community.general.imgadm:
    uuid: '*'
    state: updated

- name: Update a single image
  community.general.imgadm:
    uuid: '70e3ae72-96b6-11e6-9056-9737fd4d0764'
    state: updated

- name: Add a source
  community.general.imgadm:
    source: 'https://datasets.project-fifo.net'
    state: present

- name: Add a Docker source
  community.general.imgadm:
    source: 'https://docker.io'
    type: docker
    state: present

- name: Remove a source
  community.general.imgadm:
    source: 'https://docker.io'
    state: absent
"""

RETURN = r"""
source:
  description: Source that is managed.
  returned: When not managing an image.
  type: str
  sample: https://datasets.project-fifo.net
uuid:
  description: UUID for an image operated on.
  returned: When not managing an image source.
  type: str
  sample: 70e3ae72-96b6-11e6-9056-9737fd4d0764
state:
  description: State of the target, after execution.
  returned: success
  type: str
  sample: 'present'
"""

import re

from ansible.module_utils.basic import AnsibleModule

# Shortcut for the imgadm(8) command. While imgadm(8) supports a
# -E option to return any errors in JSON, the generated JSON does not play well
# with the JSON parsers of Python. The returned message contains '\n' as part of
# the stacktrace, which breaks the parsers.


class Imgadm:
    def __init__(self, module):
        self.module = module
        self.params = module.params
        self.cmd = module.get_bin_path("imgadm", required=True)
        self.changed = False
        self.uuid = module.params["uuid"]

        # Since there are a number of (natural) aliases, prevent having to look
        # them up every time we operate on `state`.
        if self.params["state"] in ["present", "imported", "updated"]:
            self.present = True
        else:
            self.present = False

        # Perform basic UUID validation upfront.
        if self.uuid and self.uuid != "*":
            if not re.match("^[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$", self.uuid, re.IGNORECASE):
                module.fail_json(msg="Provided value for uuid option is not a valid UUID.")

    # Helper method to massage stderr
    def errmsg(self, stderr):
        match = re.match(r"^imgadm .*?: error \(\w+\): (.*): .*", stderr)
        if match:
            return match.groups()[0]
        else:
            return "Unexpected failure"

    def update_images(self):
        if self.uuid == "*":
            cmd = [self.cmd, "update"]
        else:
            cmd = [self.cmd, "update", self.uuid]

        (rc, stdout, stderr) = self.module.run_command(cmd)

        if rc != 0:
            self.module.fail_json(msg=f"Failed to update images: {self.errmsg(stderr)}")

        # There is no feedback from imgadm(8) to determine if anything
        # was actually changed. So treat this as an 'always-changes' operation.
        # Note that 'imgadm -v' produces unparsable JSON...
        self.changed = True

    def manage_sources(self):
        force = self.params["force"]
        source = self.params["source"]
        imgtype = self.params["type"]

        cmd = [self.cmd, "sources"]

        if force:
            cmd = cmd + ["-f"]

        if self.present:
            cmd = cmd + ["-a", source, "-t", imgtype]
            (rc, stdout, stderr) = self.module.run_command(cmd)

            if rc != 0:
                self.module.fail_json(msg=f"Failed to add source: {self.errmsg(stderr)}")

            # Check the various responses.
            # Note that trying to add a source with the wrong type is handled
            # above as it results in a non-zero status.

            regex = f'Already have "{imgtype}" image source "{source}", no change'
            if re.match(regex, stdout):
                self.changed = False

            regex = f'Added "{imgtype}" image source "{source}"'
            if re.match(regex, stdout):
                self.changed = True
        else:
            # Type is ignored by imgadm(8) here
            cmd += f" -d {source}"
            (rc, stdout, stderr) = self.module.run_command(cmd)

            if rc != 0:
                self.module.fail_json(msg=f"Failed to remove source: {self.errmsg(stderr)}")

            regex = f'Do not have image source "{source}", no change'
            if re.match(regex, stdout):
                self.changed = False

            regex = f'Deleted ".*" image source "{source}"'
            if re.match(regex, stdout):
                self.changed = True

    def manage_images(self):
        pool = self.params["pool"]
        state = self.params["state"]

        if state == "vacuumed":
            # Unconditionally pass '--force', otherwise we're prompted with 'y/N'
            cmd = [self.cmd, "vacuum", "-f"]

            (rc, stdout, stderr) = self.module.run_command(cmd)

            if rc != 0:
                self.module.fail_json(msg=f"Failed to vacuum images: {self.errmsg(stderr)}")
            else:
                if stdout == "":
                    self.changed = False
                else:
                    self.changed = True
        if self.present:
            cmd = [self.cmd, "import", "-P", pool, "-q"] + ([self.uuid] if self.uuid else [])
            (rc, stdout, stderr) = self.module.run_command(cmd)

            if rc != 0:
                self.module.fail_json(msg=f"Failed to import image: {self.errmsg(stderr)}")

            regex = rf"Image {self.uuid} \(.*\) is already installed, skipping"
            if re.match(regex, stdout):
                self.changed = False

            regex = ".*ActiveImageNotFound.*"
            if re.match(regex, stderr):
                self.changed = False

            regex = f"Imported image {self.uuid}.*"
            if re.match(regex, stdout.splitlines()[-1]):
                self.changed = True
        else:
            cmd = [self.cmd, "delete", "-P", pool] + ([self.uuid] if self.uuid else [])
            (rc, stdout, stderr) = self.module.run_command(cmd)

            regex = ".*ImageNotInstalled.*"
            if re.match(regex, stderr):
                # Even if the 'rc' was non-zero (3), we handled the situation
                # in order to determine if there was a change.
                self.changed = False

            regex = f"Deleted image {self.uuid}"
            if re.match(regex, stdout):
                self.changed = True


def main():
    module = AnsibleModule(
        argument_spec=dict(
            force=dict(type="bool"),
            pool=dict(default="zones"),
            source=dict(),
            state=dict(required=True, choices=["present", "absent", "deleted", "imported", "updated", "vacuumed"]),
            type=dict(default="imgapi", choices=["imgapi", "docker", "dsapi"]),
            uuid=dict(),
        ),
        # This module relies largely on imgadm(8) to enforce idempotency, which does not
        # provide a "noop" (or equivalent) mode to do a dry-run.
        supports_check_mode=False,
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    imgadm = Imgadm(module)

    uuid = module.params["uuid"]
    source = module.params["source"]
    state = module.params["state"]

    result = {"state": state}

    # Either manage sources or images.
    if source:
        result["source"] = source
        imgadm.manage_sources()
    else:
        result["uuid"] = uuid

        if state == "updated":
            imgadm.update_images()
        else:
            # Make sure operate on a single image for the following actions
            if (uuid == "*") and (state != "vacuumed"):
                module.fail_json(msg='Can only specify uuid as "*" when updating image(s)')
            imgadm.manage_images()

    result["changed"] = imgadm.changed
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# Copyright (c) 2024, Alexei Znamensky <russoz@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:
    DOCUMENTATION = r"""
options:
  global:
    description:
      - The module passes the C(--global) argument to C(pipx), to execute actions in global scope.
    type: bool
    default: false
  executable:
    description:
      - Path to the C(pipx) installed in the system.
      - If not specified, the module uses C(python -m pipx) to run the tool, using the same Python interpreter as ansible
        itself.
    type: path
requirements:
  - This module requires C(pipx) version 1.7.0 or above.
  - Please note that C(pipx) 1.7.0 requires Python 3.8 or above.
  - Please note that C(pipx) 1.8.0 requires Python 3.9 or above.
notes:
  - This module does not install the C(pipx) python package, however that can be easily done with the module M(ansible.builtin.pip).
  - This module does not require C(pipx) to be in the shell C(PATH), but it must be loadable by Python as a module, meaning
    that C(python -m pipx) must work.
  - This module honors C(pipx) environment variables such as but not limited to E(PIPX_HOME) and E(PIPX_BIN_DIR) passed using
    the R(environment Ansible keyword, playbooks_environment).
  - This module disabled emojis in the output of C(pipx) commands to reduce clutter. In C(pipx) 1.8.0, the environment variable
    E(USE_EMOJI) was renamed to E(PIPX_USE_EMOJI) and for compatibility with both versions, starting in community.general
    11.4.0, this module sets them both to C(0) to disable emojis.
seealso:
  - name: C(pipx) command manual page
    description: Manual page for the command.
    link: https://pipx.pypa.io/latest/docs/
"""



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_datastore
version_added: "3.16.0"
short_description: Create, Update, Archive, Restore an Azure Machine Learning Datastore
description:
    - Create, Update, Delete an Azure Machine Learning Datastore.
options:
    name:
        description:
            - Name of the Azure ML datastore asset.
        type: str
        required: true
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    ml_workspace:
        description:
            - Name of the Azure ML workspace.
        required: true
        type: str
    description:
        description:
            - Description of the datastore target.
        type: str
        required: false
    resource_definition:
        description:
            - "Local path to the YAML file containing the Azure ML
              datastore specification. The YAML reference docs for
              datastore can be found at:
              https://aka.ms/ml-cli-v2-datastore-blob-yaml-reference,
              https://aka.ms/ml-cli-v2-datastore-file-yaml-reference,
              https://aka.ms/ml-cli-v2-datastore-data-lake-gen1-yaml-reference,
              https://aka.ms/ml-cli-v2-datastore-data-lake-gen2-yaml-reference."
        type: str
        required: false
    state:
        description:
            - State of the Datastore. Use C(present) to create
              or update. C(absent) to delete.
        default: present
        type: str
        choices:
            - present
            - absent

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Create ML Datastore
  azure.azcollection.azure_rm_ml_datastore:
    name: MyDataStore
    description: My Datastore created by Ansible
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace
    resource_definition: |
      FIXME.....
    tags:
      createdByToolkit: Ansible
'''

RETURN = '''
changed:
    description:
        - Whether the resource is changed.
    returned: always
    type: bool
    sample: false
ml_datastore:
    description:
        - Datastore that was just created or updated.
    returned: always
    type: dict
    sample: {
      "account_name": "workspacstoragexxxxxxxxx",
      "container_name": "datastorexxxxxxxxx-xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
      "credentials": {},
      "description": "here is a description",
      "endpoint": "core.windows.net",
      "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-datastore/providers/Microsoft.MachineLearningServices/workspaces/workspaceexxxxxxxx/datastores/datastorexxxxxxxxxx",
      "name": "datastorexxxxxxxxx",
      "protocol": "https",
      "tags": {},
      "type": "azure_blob",
    }
'''  # NOQA


try:
    import io
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
    from azure.core.exceptions import ResourceNotFoundError
    from azure.ai.ml.entities._load_functions import load_datastore
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLDatastore(MLClientCommon):

    UPDATE_OVERRIDES = ['description',
                        'tags'
                        ]

    PARAM_OVERRIDES = UPDATE_OVERRIDES + ['name']

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=True,
            ),
            description=dict(
                type='str',
                required=False,
            ),
            resource_definition=dict(
                type='str',
                required=False,
            ),
            resource_group=dict(
                type='str',
                required=True,
            ),
            ml_workspace=dict(
                type='str',
                required=True,
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent'],
            ),
        )

        self._client = None
        self.ml_registry = None
        self.state = None

        self.results = dict(
            changed=False,
            compare=[],
            ml_datastore={}
        )

        required_if = [
            ('state', 'present', ['resource_definition'])
        ]
        super(AzureRMMLDatastore, self).__init__(self.module_arg_spec,
                                                 supports_check_mode=True,
                                                 required_if=required_if)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        if self.resource_definition:
            resource_definition = io.StringIO(self.resource_definition)
        else:
            resource_definition = None

        changed = False
        ml_datastore_info = self.get(self.name)

        if self.state == 'present':
            if ml_datastore_info:
                # Update
                params_override = self.update_params(kwargs,
                                                     self.UPDATE_OVERRIDES,
                                                     ml_datastore_info)

                # Generate ml_datastore based on ansible args passed in.
                ml_datastore = load_datastore(None, params_override=params_override)
                ml_datastore_info = self.entity_to_dict(ml_datastore_info)
                changed = not self.default_compare({},
                                                   self.entity_to_dict(ml_datastore),
                                                   ml_datastore_info,
                                                   '',
                                                   self.results)
                if changed and not self.check_mode:
                    response = self.client.datastores.create_or_update(ml_datastore)
                    ml_datastore_info = self.entity_to_dict(response)

            else:
                # Create
                params_override = self.update_params(kwargs, self.PARAM_OVERRIDES)

                # Generate ml_datastore based on ansible args passed in.
                ml_datastore = load_datastore(resource_definition,
                                              params_override=params_override)
                changed = True
                if not self.check_mode:
                    response = self.client.datastores.create_or_update(ml_datastore)
                    ml_datastore_info = self.entity_to_dict(response)
        elif self.state == 'absent':
            # Delete
            if ml_datastore_info:
                changed = True
                if not self.check_mode:
                    self.delete(self.name)
                ml_datastore_info = self.get(self.name,
                                             as_dict=True)

        self.results['ml_datastore'] = ml_datastore_info
        self.results['changed'] = changed
        return self.results

    def update_params(self, kwargs, param_overrides, ml_datastore=None):
        params_override = []

        # If ml_datastore is defined use those values as a base.
        # Update can't change some of these values but we need
        # to keep the same values as before the update.
        if ml_datastore:
            items = self.entity_to_dict(ml_datastore).items()
            params_override = [{k: v} for k, v in items]

        # Update values based on what param_overrides says we can update.
        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if kwargs[key] is not None and key in param_overrides:
                params_override.append({key: kwargs[key]})
        return params_override

    def get(self, name, as_dict=False):
        try:
            # Attempt to retrieve ml_datastore based on name
            ml_datastore_info = self.client.datastores.get(name=name,
                                                           include_secrets=False)
            if as_dict:
                ml_datastore_info = self.entity_to_dict(ml_datastore_info)
        except ResourceNotFoundError:
            ml_datastore_info = None
        except Exception as e:  # pylint: disable=broad-exception-caught
            ml_datastore_info = None
        return ml_datastore_info

    def delete(self, name=None):
        try:
            response = self.client.datastores.delete(name=name)
            return response
        except Exception as e:  # pylint: disable=broad-exception-caught
            self.fail("Error delete the data instance: {0}".format(str(e)))


def main():
    AzureRMMLDatastore()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_switch_ports_by_switch_info
short_description: Information module for organizations _switch _ports _by _switch
description:
  - Get all organizations _switch _ports _by _switch.
  - List the switchports in an organization by switch.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 50. Default is 50.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
  configurationUpdatedAfter:
    description:
      - >
        ConfigurationUpdatedAfter query parameter. Optional parameter to filter items
        to switches where the configuration has been updated after the given timestamp.
    type: str
  mac:
    description:
      - >
        Mac query parameter. Optional parameter to filter items to switches with MAC
        addresses that contain the search term or are an exact match.
    type: str
  macs:
    description:
      - Macs query parameter. Optional parameter to filter items to switches that
        have one of the provided MAC addresses.
    elements: str
    type: list
  name:
    description:
      - >
        Name query parameter. Optional parameter to filter items to switches with
        names that contain the search term or are an exact match.
    type: str
  networkIds:
    description:
      - NetworkIds query parameter. Optional parameter to filter items to switches
        in one of the provided networks.
    elements: str
    type: list
  portProfileIds:
    description:
      - >
        PortProfileIds query parameter. Optional parameter to filter items to switches
        that contain switchports belonging to one of the specified port profiles.
    elements: str
    type: list
  serial:
    description:
      - >
        Serial query parameter. Optional parameter to filter items to switches with
        serial number that contains the search term or are an exact match.
    type: str
  serials:
    description:
      - Serials query parameter. Optional parameter to filter items to switches that
        have one of the provided serials.
    elements: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for switch getOrganizationSwitchPortsBySwitch
    description: Complete reference of the getOrganizationSwitchPortsBySwitch API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-switch-ports-by-switch
notes:
  - SDK Method used are
    switch.Switch.get_organization_switch_ports_by_switch,
  - Paths used are
    get /organizations/{organizationId}/switch/ports/bySwitch,
"""

EXAMPLES = r"""
- name: Get all organizations _switch _ports _by _switch
  cisco.meraki.organizations_switch_ports_by_switch_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    configurationUpdatedAfter: string
    mac: string
    macs: []
    name: string
    networkIds: []
    portProfileIds: []
    serial: string
    serials: []
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "name": "string",
      "serial": "string",
      "mac": "string",
      "network": {
        "name": "string",
        "id": "string"
      },
      "model": "string",
      "ports": [
        {
          "portId": "string",
          "name": "string",
          "tags": [
            "string"
          ],
          "enabled": true,
          "poeEnabled": true,
          "type": "string",
          "vlan": 0,
          "voiceVlan": 0,
          "allowedVlans": "string",
          "rstpEnabled": true,
          "stpGuard": "string",
          "linkNegotiation": "string",
          "accessPolicyType": "string",
          "stickyMacAllowList": [
            "string"
          ],
          "stickyMacAllowListLimit": 0
        }
      ]
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: rds_option_group_info
short_description: rds_option_group_info module
version_added: 5.0.0
description:
  - Gather information about RDS option groups.
  - This module was originally added to C(community.aws) in release 2.1.0.
author: "Alina Buzachis (@alinabuzachis)"
options:
    option_group_name:
        description:
            - The name of the option group to describe.
            - Can't be supplied together with O(engine_name) or O(major_engine_version).
        default: ''
        required: false
        type: str
    marker:
        description:
            - If this parameter is specified, the response includes only records beyond the marker, up to the value specified by O(max_records).
            - Allowed values are between V(20) and V(100).
        required: false
        type: str
    max_records:
        description:
            - The maximum number of records to include in the response.
        type: int
        default: 100
        required: false
    engine_name:
        description: Filters the list of option groups to only include groups associated with a specific database engine.
        type: str
        default: ''
        required: false
    major_engine_version:
        description:
            - Filters the list of option groups to only include groups associated with a specific database engine version.
            - If specified, then O(engine_name) must also be specified.
        type: str
        default: ''
        required: false
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: List an option group
  amazon.aws.rds_option_group_info:
    option_group_name: test-mysql-option-group
  register: option_group

- name: List all the option groups
  amazon.aws.rds_option_group_info:
    region: ap-southeast-2
    profile: production
  register: option_group
"""

RETURN = r"""
changed:
    description: True if listing the RDS option group succeeds.
    type: bool
    returned: always
    sample: false
result:
    description: The available RDS option groups.
    returned: always
    type: complex
    contains:
        allows_vpc_and_non_vpc_instance_memberships:
            description: Indicates whether this option group can be applied to both VPC and non-VPC instances.
            returned: always
            type: bool
            sample: false
        engine_name:
            description: Indicates the name of the engine that this option group can be applied to.
            returned: always
            type: str
            sample: "mysql"
        major_engine_version:
            description: Indicates the major engine version associated with this option group.
            returned: always
            type: str
            sample: "5.6"
        option_group_arn:
            description: The Amazon Resource Name (ARN) for the option group.
            returned: always
            type: str
            sample: "arn:aws:rds:ap-southeast-2:123456789012:og:ansible-test-option-group"
        option_group_description:
            description: Provides a description of the option group.
            returned: always
            type: str
            sample: "test mysql option group"
        option_group_name:
            description: Specifies the name of the option group.
            returned: always
            type: str
            sample: "test-mysql-option-group"
        options:
            description: Indicates what options are available in the option group.
            returned: always
            type: complex
            contains:
                db_security_group_memberships:
                    description: If the option requires access to a port, then this DB security group allows access to the port.
                    returned: always
                    type: complex
                    sample: list
                    elements: dict
                    contains:
                        status:
                            description: The status of the DB security group.
                            returned: always
                            type: str
                            sample: "available"
                        db_security_group_name:
                            description: The name of the DB security group.
                            returned: always
                            type: str
                            sample: "mydbsecuritygroup"
                option_description:
                    description: The description of the option.
                    returned: always
                    type: str
                    sample: "Innodb Memcached for MySQL"
                option_name:
                    description: The name of the option.
                    returned: always
                    type: str
                    sample: "MEMCACHED"
                option_settings:
                    description: The name of the option.
                    returned: always
                    type: complex
                    contains:
                        allowed_values:
                            description: The allowed values of the option setting.
                            returned: always
                            type: str
                            sample: "1-2048"
                        apply_type:
                            description: The DB engine specific parameter type.
                            returned: always
                            type: str
                            sample: "STATIC"
                        data_type:
                            description: The data type of the option setting.
                            returned: always
                            type: str
                            sample: "INTEGER"
                        default_value:
                            description: The default value of the option setting.
                            returned: always
                            type: str
                            sample: "1024"
                        description:
                            description: The description of the option setting.
                            returned: always
                            type: str
                            sample: "Verbose level for memcached."
                        is_collection:
                            description: Indicates if the option setting is part of a collection.
                            returned: always
                            type: bool
                            sample: true
                        is_modifiable:
                            description: A Boolean value that, when true, indicates the option setting can be modified from the default.
                            returned: always
                            type: bool
                            sample: true
                        name:
                            description: The name of the option that has settings that you can set.
                            returned: always
                            type: str
                            sample: "INNODB_API_ENABLE_MDL"
                        value:
                            description: The current value of the option setting.
                            returned: always
                            type: str
                            sample: "0"
                permanent:
                    description: Indicate if this option is permanent.
                    returned: always
                    type: bool
                    sample: true
                persistent:
                    description: Indicate if this option is persistent.
                    returned: always
                    type: bool
                    sample: true
                port:
                    description: If required, the port configured for this option to use.
                    returned: always
                    type: int
                    sample: 11211
                vpc_security_group_memberships:
                    description: If the option requires access to a port, then this VPC security group allows access to the port.
                    returned: always
                    type: list
                    elements: dict
                    contains:
                        status:
                            description: The status of the VPC security group.
                            returned: always
                            type: str
                            sample: "available"
                        vpc_security_group_id:
                            description: The name of the VPC security group.
                            returned: always
                            type: str
                            sample: "sg-0cd636a23ae76e9a4"
        vpc_id:
            description: If present, this option group can only be applied to instances that are in the VPC indicated by this field.
            returned: always
            type: str
            sample: "vpc-bf07e9d6"
        tags:
            description: The tags associated the RDS option group.
            type: dict
            returned: always
            sample: {
                "Ansible": "Test"
            }

"""

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.rds import get_tags
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry


@AWSRetry.jittered_backoff(retries=10)
def _describe_option_groups(client, **params):
    try:
        paginator = client.get_paginator("describe_option_groups")
        return paginator.paginate(**params).build_full_result()
    except is_boto3_error_code("OptionGroupNotFoundFault"):
        return {}


def list_option_groups(client, module):
    option_groups = list()
    params = dict()
    params["OptionGroupName"] = module.params.get("option_group_name")

    if module.params.get("marker"):
        params["Marker"] = module.params.get("marker")
        if int(params["Marker"]) < 20 or int(params["Marker"]) > 100:
            module.fail_json(msg="marker must be between 20 and 100 minutes")

    if module.params.get("max_records"):
        params["MaxRecords"] = module.params.get("max_records")
        if params["MaxRecords"] > 100:
            module.fail_json(msg="The maximum number of records to include in the response is 100.")

    params["EngineName"] = module.params.get("engine_name")
    params["MajorEngineVersion"] = module.params.get("major_engine_version")

    try:
        result = _describe_option_groups(client, **params)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't describe option groups.")

    for option_group in result["OptionGroupsList"]:
        # Turn the boto3 result into ansible_friendly_snaked_names
        converted_option_group = camel_dict_to_snake_dict(option_group)
        converted_option_group["tags"] = get_tags(client, module, converted_option_group["option_group_arn"])
        option_groups.append(converted_option_group)

    return option_groups


def main():
    argument_spec = dict(
        option_group_name=dict(default="", type="str"),
        marker=dict(type="str"),
        max_records=dict(type="int", default=100),
        engine_name=dict(type="str", default=""),
        major_engine_version=dict(type="str", default=""),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[
            ["option_group_name", "engine_name"],
            ["option_group_name", "major_engine_version"],
        ],
        required_together=[
            ["engine_name", "major_engine_version"],
        ],
    )

    # Validate Requirements
    try:
        connection = module.client("rds", retry_decorator=AWSRetry.jittered_backoff())
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS")

    results = list_option_groups(connection, module)

    module.exit_json(result=results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2017, René Moser <mail@renemoser.net>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: influxdb_write
short_description: Write data points into InfluxDB
description:
  - Write data points into InfluxDB.
author: "René Moser (@resmo)"
requirements:
  - "influxdb >= 0.9"
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  data_points:
    description:
      - Data points as dict to write into the database.
    required: true
    type: list
    elements: dict
  database_name:
    description:
      - Name of the database.
    required: true
    type: str
extends_documentation_fragment:
  - community.general._influxdb
  - community.general._attributes
"""

EXAMPLES = r"""
- name: Write points into database
  community.general.influxdb_write:
    hostname: "{{influxdb_ip_address}}"
    database_name: "{{influxdb_database_name}}"
    data_points:
      - measurement: connections
        tags:
          host: server01
          region: us-west
        time: "{{ ansible_date_time.iso8601 }}"
        fields:
          value: 2000
      - measurement: connections
        tags:
          host: server02
          region: us-east
        time: "{{ ansible_date_time.iso8601 }}"
        fields:
          value: 3000
"""

RETURN = r"""
# only defaults
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._influxdb import InfluxDb


class AnsibleInfluxDBWrite(InfluxDb):
    def write_data_point(self, data_points):
        client = self.connect_to_influxdb()

        try:
            client.write_points(data_points)
        except Exception as e:
            self.module.fail_json(msg=f"{e}")


def main():
    argument_spec = InfluxDb.influxdb_argument_spec()
    argument_spec.update(
        data_points=dict(required=True, type="list", elements="dict"),
        database_name=dict(required=True, type="str"),
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
    )

    influx = AnsibleInfluxDBWrite(module)
    data_points = module.params.get("data_points")
    influx.write_data_point(data_points)
    module.exit_json(changed=True)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2021, Marcel Zehnder (@maercu)
# Copyright: (c) 2023, Akini Ross (@akinross) <akinross@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_l3out_logical_node
short_description: Manage Layer 3 Outside (L3Out) logical node profile nodes (l3ext:RsNodeL3OutAtt)
description:
- Bind nodes to node profiles on Cisco ACI fabrics.
options:
  tenant:
    description:
    - Name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
  l3out:
    description:
    - Name of an existing L3Out.
    type: str
    aliases: [ l3out_name ]
  node_profile:
    description:
    - Name of the node profile.
    type: str
    aliases: [ node_profile_name, logical_node ]
  pod_id:
    description:
    - Existing podId.
    type: int
  node_id:
    description:
    - Existing nodeId.
    type: int
  router_id:
    description:
    - Router ID in dotted decimal notation.
    type: str
  router_id_as_loopback:
    description:
    - Configure the router ID as a loopback IP.
    type: str
    choices: [ 'yes', 'no' ]
    default: 'yes'
  loopback_address:
    description:
    - The loopback IP address.
    - The BGP-EVPN loopback IP address for Infra SR-MPLS L3Outs.
    - A configured loopback address can be removed by passing an empty string (see Examples).
    type: str
    aliases: [ loopback ]
  mpls_transport_loopback_address:
    description:
    - The MPLS transport loopback IP address for Infra SR-MPLS L3Outs.
    type: str
    aliases: [ mpls_transport_loopback ]
  sid:
    description:
    - The Segment ID (SID) Index for Infra SR-MPLS L3Outs.
    type: str
    aliases: [ segment_id ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

seealso:
- module: cisco.aci.aci_l3out
- module: cisco.aci.aci_l3out_logical_node_profile
- name: APIC Management Information Model reference
  description: More information about the internal APIC classes B(l3ext:RsNodeL3OutAtt)
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Marcel Zehnder (@maercu)
- Akini Ross (@akinross)
"""

EXAMPLES = r"""
- name: Add a new node to a node profile
  cisco.aci.aci_l3out_logical_node:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    pod_id: 1
    node_id: 111
    router_id: 111.111.111.111
    loopback_address: 111.111.111.112
    state: present
  delegate_to: localhost

- name: Add a node to a infra SR-MPLS l3out node profile
  cisco.aci.aci_l3out_logical_node: &aci_infra_node_profile_node
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: infra
    l3out: ansible_infra_sr_mpls_l3out
    node_profile: ansible_infra_sr_mpls_l3out_node_profile
    pod_id: 1
    node_id: 113
    router_id_as_loopback: 'no'
    loopback_address: 50.0.0.1
    mpls_transport_loopback_address: 51.0.0.1
    sid: 500
  delegate_to: localhost

- name: Remove a loopback address from a node in node profile
  cisco.aci.aci_l3out_logical_node:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    pod_id: 1
    node_id: 111
    loopback_address: ""
  delegate_to: localhost

- name: Delete a node from a node profile
  cisco.aci.aci_l3out_logical_node:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    pod_id: 1
    node_id: 111
    state: absent
  delegate_to: localhost

- name: Query a node
  cisco.aci.aci_l3out_logical_node:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    pod_id: 1
    node_id: 111
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all nodes
  cisco.aci.aci_l3out_logical_node:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""


from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec
from ansible.module_utils.basic import AnsibleModule


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),
        l3out=dict(type="str", aliases=["l3out_name"]),
        node_profile=dict(type="str", aliases=["node_profile_name", "logical_node"]),
        pod_id=dict(type="int"),
        node_id=dict(type="int"),
        router_id=dict(type="str"),
        router_id_as_loopback=dict(type="str", default="yes", choices=["yes", "no"]),
        loopback_address=dict(type="str", aliases=["loopback"]),
        mpls_transport_loopback_address=dict(type="str", aliases=["mpls_transport_loopback"]),
        sid=dict(type="str", aliases=["segment_id"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["tenant", "l3out", "node_profile", "pod_id", "node_id"]],
            ["state", "present", ["tenant", "l3out", "node_profile", "pod_id", "node_id"]],
        ],
        required_by={"mpls_transport_loopback_address": "loopback_address"},
        required_together=[("mpls_transport_loopback_address", "sid")],
    )

    tenant = module.params.get("tenant")
    l3out = module.params.get("l3out")
    node_profile = module.params.get("node_profile")
    pod_id = module.params.get("pod_id")
    node_id = module.params.get("node_id")
    router_id = module.params.get("router_id")
    router_id_as_loopback = module.params.get("router_id_as_loopback")
    loopback_address = module.params.get("loopback_address")
    mpls_transport_loopback_address = module.params.get("mpls_transport_loopback_address")
    sid = module.params.get("sid")
    state = module.params.get("state")

    tdn = None
    if pod_id is not None and node_id is not None:
        tdn = "topology/pod-{0}/node-{1}".format(pod_id, node_id)

    aci = ACIModule(module)

    child_classes = ["l3extLoopBackIfP"]

    if mpls_transport_loopback_address is not None:
        child_classes.append("mplsNodeSidP")

    child_configs = []

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="l3extOut",
            aci_rn="out-{0}".format(l3out),
            module_object=l3out,
            target_filter={"name": l3out},
        ),
        subclass_2=dict(
            aci_class="l3extLNodeP",
            aci_rn="lnodep-{0}".format(node_profile),
            module_object=node_profile,
            target_filter={"name": node_profile},
        ),
        subclass_3=dict(
            aci_class="l3extRsNodeL3OutAtt",
            aci_rn="rsnodeL3OutAtt-[{0}]".format(tdn),
            module_object=tdn,
            target_filter={"name": tdn},
        ),
        child_classes=child_classes,
    )

    aci.get_existing()

    if state == "present":
        if loopback_address is not None:
            if loopback_address == "" and isinstance(aci.existing, list) and len(aci.existing) > 0:
                for child in aci.existing[0].get("l3extRsNodeL3OutAtt", {}).get("children", []):
                    previous_loopback_address = child.get("l3extLoopBackIfP", {}).get("attributes", {}).get("addr")
                    child_configs.append(dict(l3extLoopBackIfP=dict(attributes=dict(addr=previous_loopback_address, status="deleted"))))
            elif loopback_address:
                loopback_address_config = dict(l3extLoopBackIfP=dict(attributes=dict(addr=loopback_address), children=[]))
                if mpls_transport_loopback_address:
                    loopback_address_config["l3extLoopBackIfP"]["children"].append(
                        dict(mplsNodeSidP=dict(attributes=dict(loopbackAddr=mpls_transport_loopback_address, sidoffset=sid)))
                    )
                child_configs.append(loopback_address_config)

        aci.payload(
            aci_class="l3extRsNodeL3OutAtt",
            class_config=dict(rtrId=router_id, rtrIdLoopBack=router_id_as_loopback, tDn=tdn),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="l3extRsNodeL3OutAtt")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()
